****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidatablest6.sas  
*
* Created (mm/dd/yyyy): 05/15/2018
* Last modified: 07/01/2019 
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:                                      
*   
*  Program inputs:  
*   -dplocal.&runid._mstr
*   -dplocal.&runid._dispensings
*                                       
*  Program outputs:    
*   -msoc.&runid._t6_utilcounts
*   -msoc.&runid._t6_utildispstats
*   -msoc.&runid._t6_utilepisdurstats
*   -msoc.&runid._t6_utilepis_censor
*   -msoc.&runid._t6_utiluptakestats
*   -msoc.&runid._t6_trendcounts
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1       07/01/19    AP        Fixed minor bugs in trendtable (DEV-4754)
*
***************************************************************************************************;

%macro ms_cidatablest6();

    %put =====> MACRO CALLED: ms_cidatablest6 v1.1;

    *Segregate levels into each table;
    data _t6levels_counts
        _t6levels_disp
        _t6levels_episdur
        _t6levels_censor
        _t6levels_uptake
        _t6levels_trend;
        set userstrata;
        
        if lowcase(tableID) = "t6counts" then output _t6levels_counts;
        if lowcase(tableID) = "t6disp" then do;
            *defensive - add DaySupp if not specified since that is an automatic strata;
            if index(lowcase(levelvars), 'daysupp') = 0 then do;
                levelvars = catx(' ',levelvars, "daysupp");
            end;
            output _t6levels_disp;
        end;
        if lowcase(tableID) = "t6episdur" then do;
            *defensive - add CumEpisodeLength if not specified since that is an automatic strata;
            if index(lowcase(levelvars), 'cumepisodelength') = 0 then do;
                levelvars = catx(' ',levelvars, "cumepisodelength");
            end;
            output _t6levels_episdur;
        end;
        if lowcase(tableID) = "t6censor" then do;
            *defensive - add EpisodeLength if not specified since that is an automatic strata;
            if index(lowcase(levelvars), 'episodelength') = 0 then do;
                levelvars = catx(' ',levelvars, "episodelength");
            end;
            output _t6levels_censor;
        end; 
        if lowcase(tableID) = "t6uptake" then do;
            *defensive - add UptakeDays if not specified since that is an automatic strata;
            if index(lowcase(levelvars), 'uptakedays') = 0 then do;
                levelvars = catx(' ',levelvars, "uptakedays");
            end;
            output _t6levels_uptake;
        end;
        if lowcase(tableID) = "t6trend" then do;
            *defensive - trend analysis must be at least on the year level. 
            if by month, month and year should be specified;
            if index(lowcase(levelvars), 'year') = 0 then do;
                levelvars = catx(' ', levelvars,'year');
            end;
            output _t6levels_trend;
        end;
    run;

    %let t6countsstrat1 = ;
    %let t6dispstrat1 = ;
    %let t6episdurstrat1 = ;
    %let t6censorstrat1 = ;
    %let t6uptakestrat1 = ;
    %let t6trendstrat1 = ;
    proc sql noprint;
        select distinct lowcase(levelvars) into: t6countsstrat1 separated by ' '
        from _t6levels_counts;
        select distinct lowcase(levelvars) into: t6dispstrat1 separated by ' '
        from _t6levels_disp;
        select distinct lowcase(levelvars) into: t6episdurstrat1 separated by ' '
        from _t6levels_episdur;
        select distinct lowcase(levelvars) into: t6censorstrat1 separated by ' '
        from _t6levels_censor;
        select distinct lowcase(levelvars) into: t6uptakestrat1 separated by ' '
        from _t6levels_uptake;
        select distinct lowcase(levelvars) into: t6trendstrat1 separated by ' '
        from _t6levels_trend;
    quit;

    %nonrep(invar = t6countsstrat1, outvar = t6countsstrat);
    %nonrep(invar = t6dispstrat1, outvar = t6dispstrat);
    %nonrep(invar = t6episdurstrat1, outvar = t6episdurstrat);
    %nonrep(invar = t6censorstrat1, outvar = t6censorstrat);
    %nonrep(invar = t6uptakestrat1, outvar = t6uptakestrat);
    %nonrep(invar = t6trendstrat1, outvar = t6trendstrat);

    ** Loop through groups **;
    %do Group=1 %to &ngroup.;

        data _null_;
            set infolder.&COHORTFILE.;
            if _n_ = &Group. then do;

                *Age stratification;
                if missing(agestrat) then do;
                    call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
                end;
                else do;
                    call symputx("agestrat",agestrat);
                end;

                call symputx("itgroup", cohortgrp);
            end;
        run;

        %let grouplist = "&itgroup."; 
        

        /****************/
        /*  UtilCounts  */
        /****************/
        %macro createutilcounts(class=,level=);

            *One record per patient for the class stratification;
            proc means data=_mstr nway noprint missing;
            var Patient IncPatient AdjustedDisp RawDisp;
            class group PatId &class.;
            output out=_num0(drop=_:) max(Patient)=NAllUsers           
                                      max(IncPatient)=NIncUsers         
                                      sum(AdjustedDisp)=AdjustedCodeCount
                                      sum(RawDisp)=RawCodeCount;           
            run;

            *One record per patient for the class stratification;
            proc means data=_num0 missing nway noprint;
            var NAllUsers NIncUsers AdjustedCodeCount RawCodeCount ;
            class group &class.;
            output out=_utilstrat(drop=_:) sum=;             
            run;

            %if %eval(&s.=1) %then %do;
                data _UtilCounts;
                    set _utilstrat;
                    format level $3.;
                    Level="&level.";
                run;
            %end;
            %else %do;
                data _UtilCounts;
                    set _UtilCounts
                        _utilstrat(in=a);
                        if a then do;
                            Level="&level.";
                        end;
                run;
            %end;

            proc datasets nowarn noprint lib=work;
                delete _utilstrat _num0;
            quit;
        %mend;

        %isdata(dataset=_t6levels_counts);    
        %if %eval(&nobs.>0) %then %do;
            %do s = 1 %to %eval(&nobs.);
                data _null_;
                    set _t6levels_counts;
                    if _n_ = &s. then do;
                        call symputx('levelid', levelid);
                        call symputx('levelvars', levelvars);
                    end;
                run;

                *Square stratifications vars;
                %ms_squaredtableshell(squarevars =&levelvars.,
                                      analysisvars=NAllUsers NIncUsers AdjustedCodeCount RawCodeCount,
                                      groups=&grouplist.,
                                      analysis=N);

                data _mstr;
                    set dplocal.&runid._mstr(in=a where=(%if %str(&grouplist.) ne () %then %do; group in (&grouplist.) and %end; INCEXL=1))
                        _mastersquare(in=b);
                    if a then do;
                        month=month(indexdt);
                        patient=1;
                        if upcase(PrevInc) = "INC" then incpatient = 1;
                            else incpatient = 0;
                    end;
                    else do;
                        patient=0;    
                        incpatient = 0;
                        AdjustedDisp = 0;
                        RawDisp = 0;
                        cens_elig = 0;
                        cens_dth = 0;
                        cens_dpend = 0;
                        cens_qryend = 0;
                        cens_episend = 0;
                    end;
                run;

                /*Summarize data */
                %createutilcounts(class=&levelvars., level=&levelid.);

                proc datasets nowarn noprint lib=work;
                    delete _mstr _mastersquare;
                quit;

            %end;

            proc sort data=_UtilCounts;
                by group level &t6countsstrat.;
            run;

            %IF %EVAL(&group.=1)  %THEN %DO;
                /*Save to MSOC*/
                data msoc.&runid._t6_utilcounts; 
                    retain group level &t6countsstrat. NAllUsers NIncUsers AdjustedCodeCount RawCodeCount;
                    set _UtilCounts;
                run;
            %END;
            %ELSE %DO;
                proc append base=msoc.&runid._t6_utilcounts
                    data=_UtilCounts(keep=group level &t6countsstrat. NAllUsers NIncUsers AdjustedCodeCount RawCodeCount) force;
                run;
            %END;

            proc datasets nowarn noprint lib=work;
                delete _UtilCounts;
            quit;
        %end;

        /*******************/
        /*  TrendCounts    */
        /*******************/

        %isdata(dataset=_t6levels_trend);
        %if %eval(&nobs.>0) %then %do;

            *Alphabetize strata and determine output name and corresponding name to give to output dataset - to faciliate merge;
            data _t6levels_trend_sorted;
                set _t6levels_trend;

                array vars[10] $150 _temporary_;
                call missing(of vars[*]);

                do i = 1 to dim(vars) until(p eq 0);
                    call scan(levelvars,i,p,l);
                    vars[i] = substrn(levelvars,p,l);
                end;
                call sortc(of vars[*]);
                length levelvars_sorted $150;
                levelvars_sorted = catx(' ',of vars[*]);
                drop i p l;

                *count number of of strata;
                levelcnt = countw(levelvars);

                format outname $12.;

                do a = 1 to levelcnt;
                    outname = cats(outname,substr(scan(levelvars_sorted, a),1,1));
                end;

                outname = upcase(cats('_', outname));

                drop a;
            run;

            ** -- Macro to calculate year-month dispensings -- **;
            %macro createdisptrend(class=,level=);
                *One record per patient for the class stratification;
                proc means data=_trenddispensings missing nway noprint;
                    var AdjDisp RawDisp;
                    class group &class.;
                    output out=_trendstrat(drop=_:) sum(AdjDisp RawDisp)=AdjustedCodeCount RawCodeCount;             
                run;

                %if %eval(&s.=1) %then %do;
                    data _trenddispstats;
                        set _trendstrat;
                        format level $3. group;
                        Level="&level.";
                    run;
                %end;
                %else %do;
                    data _trenddispstats;
                        set _trenddispstats
                            _trendstrat(in=a);
                            if a then do;
                                Level="&level.";
                            end;
                    run;
                %end;

                proc datasets nowarn noprint lib=work; delete _trendstrat; quit;
            %mend;

            * -- Macro to calculate year-month users -- **;
            *Calculate number of users based on total episode length (IndexDt --> EpisodeEndDt);

            *Assign Year/Month for each month of followup;
            data _null_;
                do i=1 to &nummonths.+1;
                    call symput("PER"||strip(put(i,best.)),put(intnx('month',&startdate.,i-1,'begin'),best.));
                end;
                do i=1 to &numyears.+1;
                    call symput("YEAR"||strip(put(i,best.)),put(intnx('year',&startdate.,i-1,'begin'),best.));
                end;
            run;

            data _mstr_fortrend; *trend analysis calculation prior to squaring;
                set dplocal.&runid._mstr(where=(group = "&itgroup." and INCEXL=1));
                patient =1;
            run;

            %macro CreateTrendSums(infile=,outfile=,by=,dimout=);

                data _temp;
                    set &infile.;
                    by &by. PatId;

                    array _NumPtsYM(*) _NumPtsYM1-_NumPtsYM&nummonths.;   *Pts level temporary vector to avoid double counting patients in same year/month;   
                    array NumPtsYM(*) NumPtsYM1-NumPtsYM&nummonths.;             *accumator of unique patient in the same year/month;
                    array _NumPtsY(*) _NumPtsY1-_NumPtsY&numyears.;   *Pts level temporary vector to avoid double counting patients in same year/month;   
                    array NumPtsY(*) NumPtsY1-NumPtsY&numyears.;                 *accumator of unique patient in the same year;

                    %do p=1 %to &nummonths.;
                        %let p1=%eval(&p.+1);
                        if %MS_PeriodsOverlap(period1=IndexDt EpisodeEndDt,period2=&&PER&p. &&PER&p1.-1) then do;
                            _NumPtsYM(&p.)=1;
                        end;                      
                    %end;

                    %do p=1 %to &numyears.;
                        %let p1=%eval(&p.+1);
                        if %MS_PeriodsOverlap(period1=IndexDt EpisodeEndDt,period2=&&YEAR&p. &&YEAR&p1.-1) then do;
                            _NumPtsY(&p.)=1;
                        end;                      
                    %end;

             
                    if last.PatId then do;
                        do i = 1 to &NUMYEARS.;               
                            NumPtsY(i)=sum(NumPtsY(i),_NumPtsY(i));
                        end;
                        do i = 1 to &nummonths.;
                             NumPtsYM(i)=sum(NumPtsYM(i),_NumPtsYM(i));
                        end; 
                        call missing(of _Num:);
                    end;

                    if last.&dimout. then do;
                        output;
                        call missing(of Num:);
                    end;

                    keep &by. Num:;

                    retain Num: _Num:;
                run;

                %ISDATA(dataset=_temp);
                %IF %EVAL(&NOBS.=0) %THEN %DO;
                    proc sql noprint;
                        insert into _temp set NumPtsYM1=.;          
                    quit;       
                %END;

                proc transpose data=_temp out=_PTS prefix=NumPts;
                    by &by.;
                run;

                data &outfile.;
                    set _PTS(rename=_NAME_=_PTS_);
                   
                    if _PTS_=:"NumPtsYM" then do;
                        per=input(tranwrd(_PTS_, 'NumPtsYM' , ''),best.);
                        date=intnx('month',&startdate.,per-1,'begin');
                        month=month(date);
                        year=year(date);
                    end;
                    else do;
                        per=input(tranwrd(_PTS_, 'NumPtsY' , ''),best.);
                        date=intnx('year',&startdate.,per-1,'begin');
                        year=year(date);
                    end;

                    rename NumPts1=NAllUsers;

                    keep year month &by. Num:;
                run;

            %mend CreateTrendSums;

            *Will calculate Sex Year Month up front because _MSY is used to further calculate Year and Month data;

                *Sex Year and Sex Year Month;
                proc sort data=_mstr_fortrend;
                    by sex patid;
                run;

                %CreateTrendSums(infile=_mstr_fortrend(drop=num:),
                                 outfile=_MSY,
                                 by=sex,
                                 dimout=Sex);

            *loop through each strata to calculate both Dispensings and User counts;
            %isdata(dataset=_t6levels_trend);
            %do s = 1 %to %eval(&nobs.);
                data _null_;
                    set _t6levels_trend_sorted;
                    if _n_ = &s. then do;
                        call symputx('levelid', levelid);
                        call symputx('levelvars', levelvars);
                        call symputx('outname', outname);
                    end;
                run;
                %put &levelvars &outname;

                *Square stratifications vars;
                %ms_squaredtableshell(squarevars =&levelvars.,
                                      analysisvars=,
                                      groups=&grouplist.,
                                      analysis=N);


                *Calculate number of dispensings based on month of dispensing date (UtilDispStats uses month of episode index);
                data _trenddispensings;
                    set dplocal.&runid._dispensings(in=a %if %str(&grouplist.) ne () %then %do;where=(group in (&grouplist.)) %end;)
                        _mastersquare(in=b);
                    if a then do;
                        month=month(adate);
                        year=year(adate);
                        AdjDisp=1;
                        RawDisp=NumDispensing;
                    end;
                    else do;
                        AdjDisp=0;
                        RawDisp=0;
                    end;
                run;

                /*Summarize dispensing data */
                %createdisptrend(class=&levelvars., level=&levelid.);

                proc datasets nowarn noprint lib=work;
                    delete _trenddispensings _mastersquare;
                quit;

                /*Summarize user data */
                    *Sex Year Month - already calculated;
                    %if "&outname" = "_MSY" | "&outname" = "_SY" %then %do;
                        data _userstrat;
                            set _MSY;
                        run;
                    %end;
                    /*Year/Year Month;=*/
                    %else %if "&outname" = "_MY" %then %do;
                        proc means data=_MSY nway noprint;
                            var NAllUsers;
                            class &levelvars.;
                            where year ne . and month ne .;
                            output out=_userstrat (drop=_:) sum(NAllUsers)=NAllUsers;
                        run;
                    %end;
                    %else %if "&outname" = "_Y" %then %do;
                        proc means data=_MSY nway noprint;
                            var NAllUsers;
                            class &levelvars.;
                            where year ne . and month = .;
                            output out=_userstrat (drop=_:) sum(NAllUsers)=NAllUsers;
                        run;
                    %end;
                    /*Sex Age Group*/
                    %else %if "&outname" = "_AASMY" | "&outname" = "_AASY" %then %do; 
                        proc sort data=_mstr_fortrend;
                            by sex agegroupnum agegroup patid;
                        run;

                        %CreateTrendSums(infile=_mstr_fortrend(drop=num:),
                                         outfile=_userstrat,
                                         by=Sex agegroupnum agegroup,
                                         dimout=agegroupnum);
                    %end;
                    /*Agegroup (age group does not vary by time)*/
                    %else %if "&outname" = "_AAMY" | "&outname" = "_AAY" %then %do; 
                        proc sort data=_mstr_fortrend;
                            By agegroupnum agegroup PatId;
                        run;
                        %CreateTrendSums(infile=_mstr_fortrend(drop=num:),
                                    outfile=_userstrat,
                                    by=agegroupnum agegroup,
                                    dimout=agegroupnum);
                    %end;
                    /*Race*/
                    %else %if %index("&outname", R) >0 %then %do;
                        proc sort data=_mstr_fortrend;
                            by race patid;
                        run;

                        %CreateTrendSums(infile=_mstr_fortrend(drop=num:),
                                         outfile=_userstrat,
                                         by=race,
                                         dimout=race);
                    %end;
                    /*hispanic*/
                    %else %if %index("&outname", H) >0 %then %do;
                        proc sort data=_mstr_fortrend;
                            by hispanic patid;
                        run;

                        %CreateTrendSums(infile=_mstr_fortrend(drop=num:),
                                         outfile=_userstrat,
                                         by=hispanic,
                                         dimout=hispanic);
                    %end;
                    %else %do;
                        %put NOTE: Unavailable stratification requested = Level &levelid.: &levelvars.;
                    %end;

                    *Determine if Month or Year/Month strata;
                    data _userstrat;
                        set _userstrat;

                        %if %index("&outname", M) > 0 %then %do;
                            if month ne .;
                        %end;
                        %else %if "&outname." ne "_Y" %then %do;
                            if month eq .;
                            drop month;
                        %end;
                    run; 

                    *Stack levels;
                    %if %eval(&s.=1) %then %do;
                        data _trenduserstat;
                            set _userstrat;
                            format level $3. group $40.;
                                Level="&levelid.";
                                group="&itgroup.";
                            run;
                    %end;
                    %else %do;
                        data _trenduserstat;
                             set _trenduserstat
                                _userstrat(in=a);
                                if a then do;
                                Level="&levelid.";
                                group="&itgroup.";
                                end;
                        run;
                    %end;

            %end; /*loop through strata*/

            proc sort data=_trenduserstat;
                by group Level &t6trendstrat.;
            run; 
            proc sort data=_trenddispstats;
                by group Level &t6trendstrat.;
            run; 

            data _trendcounts;
                merge _trenduserstat (in=a)
                      _trenddispstats (in=b);
                by group level &t6trendstrat. ;

                if not a then do;
                    nallusers=0;
                end;

                if a and missing(nallusers) then do;
                    nallusers=0;
                end;
            run;

            %IF %EVAL(&group.=1)  %THEN %DO;
                /*Save to MSOC*/
                data msoc.&runid._t6_trendcounts; 
                    retain group level &t6trendstrat. nallusers adjustedcodecount rawcodecount;
                    set _trendcounts;
                run;
            %END;
            %ELSE %DO;
                proc append base=msoc.&runid._t6_trendcounts
                    data=_trendcounts(keep=group level &t6trendstrat. nallusers adjustedcodecount rawcodecount) force;
                run;
            %END;
        %end;

        /*******************/
        /*  UtilDispStats  */
        /*******************/

        %macro createdispstats(class=,level=);

            proc means data=_alldispensings missing nway noprint;
                var DISP ;
                class group &class.;
                output out=_utilstrat(drop=_:) sum=Count;             
            run;
                
            %if %eval(&s.=1) %then %do;
                data _UtilDispStats;
                    set _utilstrat;
                    format level $3.;
                    Level="&level.";
                run;
            %end;
            %else %do;
                data _UtilDispStats;
                    set _UtilDispStats
                        _utilstrat(in=a);
                        if a then do;
                            Level="&level.";
                        end;
                run;
            %end;
        %mend;
        
        %isdata(dataset=_t6levels_disp);    
        %if %eval(&nobs.>0) %then %do;
            %do s = 1 %to %eval(&nobs.);
                data _null_;
                    set _t6levels_disp;
                    if _n_ = &s. then do;
                        call symputx('levelid', levelid);
                        call symputx('levelvars', levelvars);
                    end;
                run;

                *Square stratifications vars;
                %ms_squaredtableshell(squarevars =&levelvars.,
                                      analysisvars=,
                                      groups=&grouplist.,
                                      analysis=N);


                data _alldispensings(rename=rxsup=daysupp);
                    set dplocal.&runid._dispensings(in=a %if %str(&grouplist.) ne () %then %do;where=(group in (&grouplist.)) %end;)
                        _mastersquare(in=b);
                    if a then do;
                        month=month(indexdt);
                        DISP=1;
                    end;
                    else do;
                        DISP=0;    
                    end;
                run;
                                    
                /*Summarize data */
                %createdispstats(class=&levelvars., level=&levelid.);

                proc datasets nowarn noprint lib=work;
                    delete _alldispensings _mastersquare _utilstrat;
                quit;

            %end;

            proc sort data=_UtilDispStats;
                by group level &t6dispstrat.;
            run;

            %if %eval(&group.=1) %then %do;
                data msoc.&runid._t6_utildispstats; 
                    retain group level &t6dispstrat. count;
                    set _UtilDispStats;
                run;
            %END;
            %ELSE %DO;
                proc append base=msoc.&runid._t6_utildispstats
                    data=_UtilDispStats(keep=group level &t6dispstrat. count) force;
                run;
            %END;

            proc datasets nowarn noprint lib=work;
                delete _UtilDispstats ;
            quit;

        %end;


        /*********************/
        /*  UtilUptakeStats  */
        /*********************/

        /*Calculate uptake days - Restrict to 1st episode*/
        proc sort data=dplocal.&runid._mstr(where=(%if %str(&grouplist.) ne () %then %do; group in (&grouplist.) and %end; INCEXL=1))
            out=_mstr_index;
            by group patid indexdt;
        run;

        data _mstr_index;
            set _mstr_index;
            by group patid indexdt;
            if first.patid;

            if upcase(uptakedate) = "PRODUCTAPPROVALDATE" then do;
                uptakedays = indexdt - productapprovaldate;
            end;
            else if upcase(uptakedate) = "PRODUCTMARKETINGDATE" then do;
                uptakedays = indexdt - productmarketingdate;
            end;
            else if upcase(uptakedate) = "COMPUTEDSTARTMARKETINGDATE" then do;
                uptakedays = indexdt - computedstartmarketingdate;
            end;
            else if upcase(uptakedate) = "OTHERPRODUCTDATE" then do;
                uptakedays = indexdt - otherproductdate;
            end;
        run;

        %macro createuptakestats(class=,level=);
            proc means data=_mstr_index_squared missing nway noprint;
                var disp ;
                class group &class.;
                output out=_num01(drop=_:) sum=count;             
            run;

            %if %eval(&s.=1) %then %do;
                data _UtilUptakeStat;
                    set _num01;
                    format level $3.;
                    Level="&level.";
                run;
            %end;
            %else %do;
                data _UtilUptakeStat;
                    set _UtilUptakeStat
                        _num01(in=a);
                        if a then do;
                            Level="&level.";
                        end;
                run;
            %end;
        %mend;

        %isdata(dataset=_t6levels_uptake);    
        %if %eval(&nobs.>0) %then %do;
            %do s = 1 %to %eval(&nobs.);
                data _null_;
                    set _t6levels_uptake;
                    if _n_ = &s. then do;
                        call symputx('levelid', levelid);
                        call symputx('levelvars', levelvars);
                    end;
                run;

                *Square stratifications vars;
                %ms_squaredtableshell(squarevars =&levelvars.,
                                      analysisvars=,
                                      groups=&grouplist.,
                                      analysis=N);

                data _mstr_index_squared;
                    set _mstr_index(in=a)
                        _mastersquare(in=b);
                    if a then do;
                        month=month(indexdt);
                        DISP=1;
                    end;
                    else do;
                        DISP=0;    
                    end;
                run;

                /*Summarize data */
                %createuptakestats(class=&levelvars., level=&levelid.);

                proc datasets nowarn noprint lib=work;
                    delete _mstr_index_squared _mastersquare _num:;
                quit;
            %end;

            proc sort data=_UtilUptakeStat;
                by group level &t6uptakestrat.;
            run;

            %if %eval(&group.=1) %then %do;
                data msoc.&runid._t6_utiluptakestats; 
                    retain group level &t6uptakestrat. count;
                    set _UtilUptakeStat;
                run;
            %END;
            %ELSE %DO;
                proc append base=msoc.&runid._t6_utiluptakestats
                    data=_UtilUptakeStat(keep=group level &t6uptakestrat. count) force;
                run;
            %END;

            proc datasets nowarn noprint lib=work;
                delete _UtilUptakeStat;
            quit;

        %end;

        /**********************/
        /*  UtilEpisDurStats  */
        /**********************/

        %macro createepisdurcounts(class=,level=);

            proc means data=_mstr nway noprint missing;
                var Patient ;
                class group PatId &class.;
                output out=_num01(drop=_:) max=;    
            run;

            *One record per patient for the class stratification;
            proc means data=_num01 missing nway noprint;
                var Patient;
                class group &class.;
                output out=_num01(drop=_:) sum=npts;             
            run;

            %if %eval(&s.=1) %then %do;
                data _UtilEpisDurStats;
                    set _num01;
                    format level $3. group $40.;
                    Level="&level.";
                    if npts = . then npts = 0;
                run;
            %end;
            %else %do;
                data _UtilEpisDurStats;
                    set _UtilEpisDurStats
                        _num01(in=a);
                        if a then do;
                            Level="&level.";
                            if npts = . then npts = 0;
                        end;           
                run;
            %end;
        %mend;

        %isdata(dataset=_t6levels_episdur);    
        %if %eval(&nobs.>0) %then %do;
            %do s = 1 %to %eval(&nobs.);
                data _null_;
                    set _t6levels_episdur;
                    if _n_ = &s. then do;
                        call symputx('levelid', levelid);
                        call symputx('levelvars', levelvars);
                    end;
                run;

                *Square stratifications vars;
                %ms_squaredtableshell(squarevars =&levelvars.,
                                      analysisvars=,
                                      groups=&grouplist.,
                                      analysis=N);

                data _mstr;
                    set dplocal.&runid._mstr(in=a where=(%if %str(&grouplist.) ne () %then %do; group in (&grouplist.) and %end; INCEXL=1))
                        _mastersquare(in=b);
                    if a then do;
                        month=month(indexdt);
                        patient=1;
                    end;
                    else do;
                        patient=0;    
                    end;
                run;

                /*Summarize data */
                %createepisdurcounts(class=&levelvars., level=&levelid.);

                proc datasets nowarn noprint lib=work;
                    delete _mstr _mastersquare _num:;
                quit;

            %end;

            proc sort data=_UtilEpisDurStats;
                by group level &t6episdurstrat.;
            run;

            %if %eval(&group.=1) %then %do;
                data msoc.&runid._t6_utilepisdurstats; 
                    retain group level &t6episdurstrat. npts;
                    set _UtilEpisDurStats;
                run;
            %end;
            %else %do;
                proc append base=msoc.&runid._t6_utilepisdurstats
                    data=_UtilEpisDurStats(keep=group level &t6episdurstrat. npts) force;
                run;
            %end;

        %end;

        /**********************/
        /*  UtilEpis_Censor   */
        /**********************/

        %macro createcensorcounts(class=,level=);

            proc means data=_mstr nway noprint missing;
                var Patient;
                class group &class.;
                output out=_num0(drop=_:) sum(cens_elig)=EndEnrollmentCount         
                                          sum(cens_dth)=DeathCount  
                                          sum(cens_dpend)=EndAvailDataCount  
                                          sum(cens_qryend)=EndQueryCount  
                                          sum(cens_episend)=EndProductDiscontinuationCount;    
            run;

            %if %eval(&s.=1) %then %do;
                data _utilepis_censor;
                    set _num0;
                    format level $3.;
                    Level="&level.";
                run;
            %end;
            %else %do;
                data _utilepis_censor;
                    set _utilepis_censor
                        _num0(in=a);
                    if a then do;
                        Level="&level.";
                    end;           
                run;
            %end;
        %mend;

        %isdata(dataset=_t6levels_censor);    
        %if %eval(&nobs.>0) %then %do;
            %do s = 1 %to %eval(&nobs.);
                data _null_;
                    set _t6levels_censor;
                    if _n_ = &s. then do;
                        call symputx('levelid', levelid);
                        call symputx('levelvars', levelvars);
                    end;
                run;

                *Square stratifications vars;
                %ms_squaredtableshell(squarevars =&levelvars.,
                                      analysisvars=,
                                      groups=&grouplist.,
                                      analysis=N);

                data _mstr;
                    set dplocal.&runid._mstr(in=a where=(%if %str(&grouplist.) ne () %then %do; group in (&grouplist.) and %end; INCEXL=1))
                        _mastersquare(in=b);
                    if a then do;
                        month=month(indexdt);
                        patient=1;
                    end;
                    else do;
                        patient=0;    
                        cens_elig=0;
                        cens_dth=0;
                        cens_dpend=0;
                        cens_qryend=0;
                        cens_episend=0;
                    end;
                run;

                /*Summarize data */
                %createcensorcounts(class=&levelvars., level=&levelid.);

                proc datasets nowarn noprint lib=work;
                    delete _mstr _mastersquare;
                quit;

            %end;

            proc sort data=_utilepis_censor;
                by group level &t6censorstrat.;
            run;

            %IF %EVAL(&group.=1)  %THEN %DO;
                /*Save to MSOC*/
                data msoc.&runid._t6_utilepis_censor; 
                    retain group level &t6censorstrat. endenrollmentcount deathcount endavaildatacount endquerycount endproductdiscontinuationcount;
                    set _utilepis_censor;
                run;
            %END;
            %ELSE %DO;
                proc append base=msoc.&runid._t6_utilepis_censor
                            data=_utilepis_censor(keep=group level &t6censorstrat. 
                                            endenrollmentcount deathcount endavaildatacount endquerycount endproductdiscontinuationcount) force;
                run;
            %END;

            proc datasets nowarn noprint lib=work;
                delete _utilepis_censor;
            quit;
        %end;


    %end; /*loop through groups*/

    %put NOTE: ******** END OF MACRO: ms_cidatablest6 v1.1 ********;

%mend ms_cidatablest6;
