****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_create_enrollment_spans.sas  
*
* Created (mm/dd/yyyy): 11/10/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   Creates enrollment spans for each scenario in CIDA                                    
*   
*  Program inputs:                                                                                   
* 
*  Program outputs:                                                                                                                                       
* 
*  PARAMETERS:                                                                       
*
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_create_enrollment_spans();

%put =====> MACRO CALLED: ms_create_enrollment_spans v1.0;

	proc sql noprint;
		select max(enrollmentnum) into: numenrollgroup
		from cohortfile_for_enr;
	quit;

	%put total number of enrollment datasets created: &numenrollgroup.;

	%do enr =1 %to &numenrollgroup.;

		data _null_;
			set cohortfile_for_enr(where=(enrollmentnum = &enr.));
			 if _n_ =1 then do;

			 	*Type of analysis;
				if upcase(type1)=:"Y" then call symputx('Type',1);
				if upcase(type2)=:"Y" then call symputx('Type',2);
				if upcase(type3)=:"Y" then call symputx('Type',3);
				if upcase(type4)=:"Y" then call symputx('Type',4);
				if upcase(type5)=:"Y" then call symputx('Type',5);
				if upcase(type6)=:"Y" then call symputx('Type',6);

				*Enrollment parameters;
				call symputx("COVERAGE",strip(lowcase(coverage)));
						
				*Enrollment gap;
				if enrolgap ne . then call symputx("ENROLGAP",enrolgap);
				else call symputx("ENROLGAP","0"); 

				*Compound name for suffix of elig file;
				if missing(CHARTRES) then CHARTRES='N';
				call symputx("CHARTRES",lowcase(CHARTRES));
				call symputx("RECONTYP",strip(lowcase(coverage))||compress(CHARTRES,"_"));

				*death;
				if upcase(censor_dth) eq "Y" then call symputx("censor_dth",1);
				else call symputx("censor_dth",0);

				*Demographic criteria;
					*Sex values applicable to demographics;
					if missing(Sex) then do;
					call symputx("DemogSex","'F' 'M' 'U' 'A'");
					end;
					else do;
					call symputx('DemogSex',upcase(Sex));
					end;

					*Race values applicable to demographics;
					if missing(Race) then do;
					call symputx("DemogRace","'0' '1' '2' '3' '4' '5'");
					end;
					else do;
					call symputx("DemogRace",upcase(Race));
					end;

					*Hispanic values applicable to demographics;
					if missing(Hispanic) then do;
					call symputx("DemogHispanic","'Y' 'N' 'U'");
					end;
					else do;
					call symputx("DemogHispanic",upcase(Hispanic));
					end;	
			end;
		run;
		%PUT &COVERAGE. &CHARTRES. &ENROLGAP. &RECONTYP. &demogsex. &demograce. &demoghispanic.;   

		*Add Type 3 info for death censoring;
		%if %eval(&Type. = 3) %then %do;    
	    	data _CohortDef;
		   	 	set infolder.&TYPE3FILE.;
				if _n_ =1 then do;
					if T3ENDOFUPDATE=. then T3ENDOFUPDATE=99999;
		    		call symputx("T3ENDOFUPDATE",T3ENDOFUPDATE);

				end;
	    	run;
		%end;

		*Create enrollment spans;
 	    %MS_EPISODEREC2(INFILE=indata.&ENRTABLE.,         		  /*SAS dataset containing the raw enrollment data*/
	                    OUTFILE=dplocal.enr_&RECONTYP.&ENROLGAP., /*SAS output dataset containing the conciliated enrollment periods*/
	                    COVERAGE=&COVERAGE.,              		  /*Drug/Medical coverage indicator*/
	                    CHARTRES=&CHARTRES.,                  	  /*Chart availability indicator*/
	                    ENRSTART=Enr_Start,               		  /*Name of enrollment episode start date variable in INFILE*/
	                    ENREND=Enr_End,                   		  /*Name of enrollment episode end date variable in INFILE*/
	                    ENROLGAP=&ENROLGAP.,              		  /*Maximum allowable enrollment episode gap*/
	                    REMOVEDUP=Yes);  	              		  /*Indicates the removal of duplicates from the output file*/

		* Delete data for patients in DPs &PTSTOEXCLUDE. file; 
		%ms_delpatients(datafile=dplocal.enr_&RECONTYP.&ENROLGAP.,
		                ptsfile=&PTSTOEXCLUDE.,
		                Outfile=dplocal.enr_&RECONTYP.&ENROLGAP.);

		*Merge with demographic data;
		proc sql noprint undo_policy=none;
			create table dplocal.enr_&RECONTYP.&ENROLGAP._dem as
			select Enr.*,
				   Dem.BIRTH_DATE,
				   Dem.Sex,
				   Dem.Race,
				   Dem.Hispanic,
				   Dem.zip,
				   Dem.zip_date
			from dplocal.enr_&RECONTYP.&ENROLGAP. as Enr,
				 indata.&DEMTABLE. as Dem
			where Enr.patid=Dem.patid and upcase(Dem.sex) in (&DemogSex.) and 
			upcase(Dem.Race) in (&DemogRace.) and upcase(Dem.Hispanic) in (&DemogHispanic.) 
			order by PatID, Enr_Start, Enr_End;
		quit;

		* Delete data for patients in DPs &PTSTOEXCLUDE. file; 
		%ms_delpatients(datafile=dplocal.enr_&RECONTYP.&ENROLGAP._dem,
	                	ptsfile=&PTSTOEXCLUDE.,
	                	Outfile=dplocal.enr_&RECONTYP.&ENROLGAP._dem);

		*Need to save for Type 4;
		%if %eval(&Type. ne 4) %then %do;    
		proc datasets nowarn noprint lib=dplocal;
			delete enr_&RECONTYP.&ENROLGAP.;
		quit;
		%end;
	
		*Merge with death data;
		data dplocal.enr_&enr.;
			merge dplocal.enr_&RECONTYP.&ENROLGAP._dem(in=a)
		          dplocal.deathextract(in=b);	*Already sorted by PatID;
		by PatId;
		if a;
		enr_end_equal_death = 0;
		if &censor_dth.=1 then do;
			if b and DeathDt <= Enr_End then do;
				EnrEpisodeCensoredAtDeath=1;
				if enr_end = deathdt then enr_end_equal_death = 1; /*For censoring table*/
				if Enr_end < Enr_Start then delete;
			end;
		end;
	    %if %eval(&type. = 3) %then %do;
			if EnrEpisodeCensoredAtDeath=1 then Enr_End=DeathDt; /*Adjust for Type 3, keep for other types and adjust later*/
/*			if &censordate. < Enr_End then Enr_End=&censordate.;*/
			if Enr_end < Enr_Start then delete;
	    %end;

		*recode A/U sex values;
		if sex in ('A', 'U') then sex = 'O';
		run;	

		proc datasets nowarn noprint lib=dplocal;
			delete enr_&RECONTYP.&ENROLGAP._dem;
		quit;

	%end;

	%put NOTE: ********END OF MACRO: ms_create_enrollment_spans v1.0********;

%mend ms_create_enrollment_spans;
