****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createmicohorts.sas  
*
* Created (mm/dd/yyyy): 10/20/2018
* Last modified: 06/30/2019
* Version: 1.2
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Create exposure and control (unexposed, comparator) cohorts, process according to MICOHORTFILE
*	including additional inclusion/exclusion criteria, and evaluate outcomes 
*   
*  Program inputs:  
*             	                        
*  Program outputs:    
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  02/12/19	  AP		DEV-4163: Added OutcomeFromAnchor parameter
*
*   1.2		  06/30/19	  AP		QRP 8.0.0 - integration of tree extraction
*									Changed Analysisgrp to MILGrp
*									Added run time metrics
*
***************************************************************************************************;

%macro ms_createmicohorts();

	%put =====> MACRO CALLED: ms_createmicohorts v1.2; 

	%isdata(dataset=infolder.&MICOHORTFILE.);
	%if %eval(&nobs.>0) %then %do;
		%let n= &nobs.;

		/*loop through milgrp*/
		%do a=1 %to &n.;

			%ms_starttimer(timestart=micohortruntime);

			*Cohort file parameters;
			data _null_;
				set infolder.&MICOHORTFILE.;
				if _n_ = &a. then do;
					call symputx('milgrp', lowcase(milgrp));
					call symputx('groupname', lowcase(groupname));
					call symputx('expmp', upcase(expmp));
					if missing(controlmp) = 0 then do;
						call symputx('controlmp', upcase(controlmp));
					end;
					else do;
						call symputx('controlmp', '');
					end;

					*Infant sex values applicable to demographics;
					if missing(csex) then do;
						call symputx("Restrictcsex", "N");
						call symputx("CDemogSex","'F' 'M' 'O'");
					end;
					else do;
						call symputx("Restrictcsex", "Y");
						call symputx('CDemogSex',upcase(csex));
					end;

					call symputx('exposureunit', upcase(exposureunit));
					call symputx('exposurefrom', exposurefrom);
					call symputx('exposureto', exposureto);
					call symputx('outcomepop', upcase(outcomepop));
					call symputx('indexdate', upcase(indexdate));
					call symputx('outcomefromanchor', upcase(outcomefromanchor));
					call symputx('outcometoanchor', upcase(outcometoanchor));
					call symputx('outcomefrom', outcomefrom);
					call symputx('outcometo', outcometo);
				end;
			run;

			data _null_;
				set infolder.&COHORTFILE.;
				where lowcase(cohortgrp) = "&groupname";         
					*Age stratification;
					if missing(agestrat) then do;
						call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
					end;
					else do;
						call symputx("agestrat",agestrat);
					end;
			run;

			data _null_;
				set infolder.&type4file.;
				where lowcase(group) = "&groupname";         
				*Pre pregnancy period;
				if missing(prepregdays) then call symputx('prepregdays', 0);
				else call symputx('prepregdays', abs(prepregdays));
			run;

			/* For data-driven query period, determine query period */
			%if "&datadrivenqueryperiod" = "Y" %then %do;

				*Confirm outcome anchors set correctly;
				%if %str("&outcomefromanchor") ne "INDEXDT_DELIV" | %str("&outcometoanchor") ne "INDEXDT_DELIV" %then %do;
					%put ERROR: Outcome anchors must be set to Delivery Date for data driven query periods;
					%abort;
				%end; 

				data &monitoringfile.&a.;
					set &monitoringfile.(drop=enddate);

					*Add this MILGRP end date to monitoring file;
					enddate = INTNX('Month', censordate-max(&outcometo.,0, &enrdaysaftInd.),0, 'begin')-1;
					
					format enddate date9.;
					call symputx('enddate', enddate);
				run;

				*Recreate look dataset;
				proc sort data=&Monitoringfile.&a. out=looks;
	        		by PERIODID;
	   			run;
	
			    data looks;
			        set looks;
			        format lenddate LookStart LookEnd date9.;
			        LookStart=startdate;
			        LookEnd=enddate;
			        if periodid>1 then LookStart=lenddate+1;
			        lenddate=enddate;
			        retain lenddate;
			        rename PeriodId = Look;
			        keep PeriodId LookStart LookEnd;
				run; 

				*Output METADATA file;
				data _metadata;
					retain reqid runid group timeperiod startdate expperiodstartdt expperiodenddt survenddate;
					merge &monitoringfile.&a.(keep=startdate censordate periodid rename=censordate =survenddate rename=periodid=timeperiod where=(timeperiod=&periodidstart.))
						looks(keep=look lookstart lookend where=(timeperiod=&periodidstart.) 
						rename=look=timeperiod rename=lookstart=expperiodstartdt rename=lookend = expperiodenddt);
					by timeperiod;

					format reqid $40. runid $3. group $40. startdate expperiodstartdt expperiodenddt survenddate date9.;
					length reqid $40 runid $3 group $40;

					reqid = "&reqid.";
					runid = "&runid.";
					group = "&milgrp.";
				run;

				%if %eval(&a.=1) %then %do;
					data msoc.&runid._metadata_for_time_period_&PERIODIDSTART.;
						set _metadata;
					run;
				%end;
				%else %do;
					proc append base=msoc.&runid._metadata_for_time_period_&PERIODIDSTART.
						data=_metadata force;
					run;
				%end;
			%end;

			*Custom warning if control cohort is unexposed and index date is INDEXDT_EXP;
			%if %str(&controlmp) = %str() and "&indexdate" = "INDEXDT_EXP" %then %do;
				%put WARNING: User defined index date is set to exposure date, however control cohort is unexposed;
			%end;

			*Custom warning if control cohort is unexposed and outcome anchor is INDEXDT_EXP;
			%if %str(&controlmp) = %str() and ("&outcomefromanchor" = "INDEXDT_EXP" | "&outcometoanchor" = "INDEXDT_EXP") %then %do;
				%put WARNING: User defined outcome anchor date is set to exposure date, however control cohort is unexposed;
			%end;

			*Create dataset that contains valid group names;
			%if %eval(&a.=1) %then %do;
				data migroupnames;
					length cohortgrp $40.;
					cohortgrp = cats("&milgrp","_eoi"); output;
					cohortgrp = cats("&milgrp","_ref"); output;
				run;
			%end;
			%else %do;
				data migroupnames;
					set migroupnames end=eof;
					output;
					if eof then do;
						cohortgrp = cats("&milgrp","_eoi"); output;
						cohortgrp = cats("&milgrp","_ref"); output;
					end;
				run;
			%end;

			*Create new cohorts;
				*Exposure cohort;
				data _preg_eoi;
					set dplocal.&runid._concepi_mstr_preg_tri(where=(lowcase(group)="&groupname" and moiname = "&expmp."));

					if "&exposureunit" = "T" then do;
						do i=&exposurefrom. to &exposureto. until (exposuretri=1);
							if i = -1 then do;
								if usepre = 1 then exposuretri = 1;
							end;
							else if i = 1 then do;
								if anyt1 = 1 then exposuretri = 1;
							end;
							else if i = 2 then do;
								if anyt2 = 1 then exposuretri = 1;
							end;
							else if i = 3 then do;
								if anyt3 = 1 then exposuretri = 1;
							end;
						end;

					end;
					else if "&exposureunit" = "W" then do;
						if gestwkend >=&exposurefrom. and gestwk <=&exposureto. then exposuretri=1;
					end;

					*Only keep if infant is the desired sex;
					csexmatch = 1;
					%if "&Restrictcsex" = "Y" %then %do;
						if csex not in (&CDemogSex.) then csexmatch = 0;
					%end;

					if exposuretri=1 & csexmatch = 1 then output;
				run;

				*Control cohort;
				data _preg_ref;
					set dplocal.&runid._concepi_mstr_preg_tri(where=(lowcase(group)="&groupname"));

					exposuretri = 0;

					*Only keep if infant is the desired sex;
					csexmatch = 1;
					%if "&Restrictcsex" = "Y" %then %do;
						if csex not in (&CDemogSex.) then csexmatch = 0;
					%end;

					%if %str(&controlmp.) ne %str() %then %do;
						*comparator group;
						if moiname = "&controlmp." then do;
							if "&exposureunit" = "T" then do;
								do i=&exposurefrom. to &exposureto. until (exposuretri=1);
									if i = -1 then do;
										if usepre = 1 then exposuretri = 1;
									end;
									else if i = 1 then do;
										if anyt1 = 1 then exposuretri = 1;
									end;
									else if i = 2 then do;
										if anyt2 = 1 then exposuretri = 1;
									end;
									else if i = 3 then do;
										if anyt3 = 1 then exposuretri = 1;
									end;
								end;
							end;
							else if "&exposureunit" = "W" then do;
								if gestwkend >=&exposurefrom. and gestwk <=&exposureto. then exposuretri=1;
							end;

							*output;
							if exposuretri=1 & csexmatch = 1 then output;
						end;
					%end;
					*Unexposed group;
					%if %str(&controlmp.) = %str() %then %do;
						if moiname = "&expmp." then do;
							if "&exposureunit" = "T" then do;
								do i=&exposurefrom. to &exposureto. until (exposuretri=1);
									if i = -1 then do;
										if usepre = 1 then exposuretri = 1;
									end;
									else if i = 1 then do;
										if anyt1 = 1 then exposuretri = 1;
									end;
									else if i = 2 then do;
										if anyt2 = 1 then exposuretri = 1;
									end;
									else if i = 3 then do;
										if anyt3 = 1 then exposuretri = 1;
									end;
								end;

							end;
							else if "&exposureunit" = "W" then do;
								if gestwkend >=&exposurefrom. and gestwk <=&exposureto. then exposuretri=1;
							end;

							*output;
							if csexmatch = 1 then output;
						end;
						else do;
							exposuretri = 0;
							if csexmatch = 1 then output;
						end;
					%end;
				run;

				%if %str(&controlmp.) = %str() %then %do;

				*because dplocal.&runid._concepi_mstr_preg_tri contains 1 row per MOI, 
					 need to roll up to 1 row per delivery to ensure delivery unexposed during exposure period;
					proc means data=_preg_ref noprint nway;
						var exposuretri;
						class patid indexdt;
						output out=_preg_ref_1perdeliv(drop=_:) max=exposuredeliv;
					run;

					proc sql noprint;
						create table _preg_ref1 as
						select ref.*
							 , d.exposuredeliv
						from _preg_ref as ref 
						inner join _preg_ref_1perdeliv as d
						on ref.patid = d.patid and ref.indexdt = d.indexdt
						where d.exposuredeliv ne 1
						order by ref.patid, ref.indexdt;
					quit;

					*only 1 delivery per unexposed cohort;
					data _preg_ref;	
						set _preg_ref1(drop=exposuredeliv);
						by patid indexdt;
						if first.indexdt;

						*recode moiname to expMP to correctly count other period exposures;
						if moiname ne "&expmp." then do;
							indexdt2 = .;
							episodeenddt2 = .;
							gestwkend = .;
						end;
						moiname = "&expmp.";
					run;
				%end;

				*Determine maximum treewashper;
				%if "&analysis" = "tree" %then %do;
					%isdata(dataset=infolder.&comparison.);
					%if %eval(&nobs.>0) %then %do;
						*Get analysisgrp;
						proc sql noprint;
							select quote(lowcase(strip(analysisgrp))) into: analysisgrplist separated by ','
							from infolder.&comparison.
							where lowcase(eoi)= lowcase("&milgrp._eoi") | lowcase(ref)= lowcase("&milgrp._ref");
						quit;

						proc sql noprint;
							select max(treewashper) into: maxtreewashper
							from infolder.&treefile.
							where lowcase(group) in (&analysisgrplist);
						quit;
					%end;
					%else %do;
						%put ERROR: Comparison file is missing. This is a required input file when using tree extraction;
						%abort;
					%end;
				%end;

				*Merge exposure and comparator cohorts;
				data _micohorts(drop=MinEnrolMet csexmatch %if "&analysis" = "tree" %then %do; IndexDtLookStart IndexDtLookEndDt IndexLook %end;);
					set _preg_eoi(in=a drop=group rename=indexdt=pregstartdt rename=episodeenddt = pregenddt)
						_preg_ref(in=b drop=group rename=indexdt=pregstartdt rename=episodeenddt = pregenddt);

					format group milgrp $40. indexdate outcometoanchor outcomefromanchor $20. indexdt date9. outcomepop $2.;

					milgrp = "&milgrp";
					if a then group = cats("&milgrp","_eoi");
					if b then group = cats("&milgrp","_ref");
					type = 4;

					*set indexdt;
					if "&indexdate" = "INDEXDT" then do;
						indexdt = pregstartdt;
						indexdate = "INDEXDT";
					end;
					if "&indexdate" = "INDEXDT_EXP" then do;
						indexdt = indexdt2;
						indexdt_exp = indexdt2;
						indexdate = "INDEXDT_EXP";

						*Reassign index date if begins prior to exposure from period;
						*define exosurefrom date;
						if "&exposureunit" = "T" then do;
							if &exposurefrom. = -1 then do;
								exposurefromdt = pregstartdt - &prepregdays.;
							end;
							else if &exposurefrom. = 1 then do;
								exposurefromdt = pregstartdt;
							end;
							else if &exposurefrom. = 2 then do;
								exposurefromdt = pregstartdt+90;
							end;
							else if &exposurefrom. = 3 then do;
								exposurefromdt = pregstartdt+180;
							end;
						end;
						if "&exposureunit" = "W" then do;
							if &exposurefrom. <0 then exposurefromdt = pregstartdt + ((int(((&exposurefrom.+1)*7)-1))-6);
							else if &exposurefrom. >=0 then exposurefromdt = pregstartdt + (int(((&exposurefrom.)*7)+1)-8);
						end;

						if indexdt < exposurefromdt then do;
							indexdt = exposurefromdt;
							indexdt_exp = exposurefromdt;
						end;
					end;
					if "&indexdate" = "INDEXDT_DELIV" then do;
						indexdt = pregenddt;
						indexdate = "INDEXDT_DELIV";
					end;

					*Assign ID to delivery in order to link Mothers-Infants during Incl/Excl processing;
					milID = _n_;

					/*Compute difference between date of enrollment and date of  birth for linked infants*/
					enroll_diff = cenr_start - cbirth_date;

					/*Compute length of enrollment for matched infants*/
					birth_enroll = cenr_end - cenr_start;

					/*Dates for Outcome evaluation*/
					format outcomestart outcomeend date9.;
					if "&outcomefromanchor" = "INDEXDT" then do;
						outcomestart = pregstartdt + &outcomefrom.;
					end;
					if "&outcomefromanchor" = "INDEXDT_EXP" then do;
						outcomestart = indexdt2 + &outcomefrom.;
					end;
					if "&outcomefromanchor" = "INDEXDT_DELIV" then do;
						outcomestart = pregenddt + &outcomefrom.; 
					end;
					outcomestart = indexdt + &outcomefrom.;
					if "&outcometoanchor" = "INDEXDT" then do;
						outcomeend = pregstartdt + &outcometo.;
					end;
					if "&outcometoanchor" = "INDEXDT_EXP" then do;
						outcomeend = indexdt2 + &outcometo.;
					end;
					if "&outcometoanchor" = "INDEXDT_DELIV" then do;
						outcomeend = pregenddt + &outcometo.; 
					end;

					*Re-evaluate enrollment for baseline covariates using new index date;
					if Enr_Start <= indexdt -max(0,&MinCovFrom.,&L1HDPSFROM.) then MinEnrolMet=1; 

					*Evaluate Treewash enrollmenet from outcome start date;
					if Enr_Start <= outcomestart - max(0, &maxtreewashper.) then MinEnrolMet2 = 1;

					*analysis specific parameter values;
					exposureunit = "&exposureunit";
					exposurefrom = &exposurefrom.;
					exposureto = &exposureto.;
					outcomepop = "&outcomepop";
					outcomefromanchor = "&outcomefromanchor";
					outcometoanchor = "&outcometoanchor";
					outcomefrom = &outcomefrom.;
					outcometo = &outcometo.;

					if MinEnrolMet and MinEnrolMet2;

					*For ADS;
					LastLookFollowed = .;

					drop age:;

					*compute followuptime and ttc for treescan;
					%if "&analysis" = "tree" %then %do;
						followuptime = min(outcomeend, enr_end) - pregenddt + 1;
						if Enr_End <= outcomeend then do;
						ttc = Enr_End - pregenddt;
					end;
					%end;
			run;
			
			*Tree extraction - delivery date must meet look periods;
			%if "&analysis" = "tree" %then %do;
				proc sql noprint undo_policy=none;
			        create table _micohorts as
			        select dat.*,
						   look.lookstart as IndexDtLookStart,
			               look.LookEnd as IndexDtLookEndDt,
			               Look.Look as IndexLook
			        from _micohorts as dat,
			             looks as look
			        where look.LookStart<=dat.pregenddt<=look.LookEnd;
				quit;
			%end;

			*Reassess Age Group criteria, zip_uncertain criteria;
			%ms_agestrat(infile=_micohorts,
		                 outfile=_micohorts,
		                 startdt=birth_date,
		                 enddt=indexdt,
		                 timestrat=&agestrat.);

			data _micohorts;
		    	set _micohorts;
				if strip(AgeGroup) ne "";
				%if "&geog." = "Y" %then %do;
					if missing(zip_date) or indexdt < zip_date then Zip_uncertain = 'Y';
					else zip_uncertain = 'N';
				%end;
				%else %do;
					zip_uncertain = '';
				%end;

				age = (indexdt - birth_date)/365.25;
			run;

			*list of mother and infant IDs;
			data _mi_ids_enr_eoi
			     _mi_ids_enr_ref;
				set _micohorts(keep=patid enr_start enr_end group)
				    _micohorts(keep=linkedid cenr_start cenr_end group rename=linkedid=patid rename=cenr_start=enr_start rename=cenr_end=enr_end
								where=(missing(patid)=0));
					
				if group = "&milgrp._eoi" then output _mi_ids_enr_eoi;
				if group = "&milgrp._ref" then output _mi_ids_enr_ref;
			run;

			proc sort data=_mi_ids_enr_eoi(keep=patid) nodupkey out=PtsList_eoi;
				by patid;
			run;
			proc sort data=_mi_ids_enr_ref(keep=patid) nodupkey out=PtsList_ref;
				by patid;
			run;

			/**************************************************************/
			/* Process additional codes: Inclusion/Exclusion and outcomes */
			/**************************************************************/

			*Evaluate separately for EXP and COMP cohorts;

			%macro mi_pov3_5(groupind=);

				proc datasets nowarn noprint lib=work;
					delete _diag _others _ITGroupParamNDC _ITGroupParamMeds _ITGroupParamLabs;
				run;

				data _diag
					 _others;
					set &cohortcodes.;
					where ( (T4_INDEX in('INC','EXC')) | T4_FUP = 'DEF') and lowcase(group) in ("&milgrp.&groupind.");
				run;

				%isdata(dataset=_diag);
				%if %eval(&nobs.>=1) %then %do;
					%MS_ProcessWildcards(InFile=_diag, 
						                 CodeVar=code, 
						                 OutFile=_diag);
				%end;

				data _ITGroupParamNDC
					 _ITGroupParamMeds
					 _ITGroupParamLabs(rename=Group=RawGroup rename=CareSettingprincipal=RawCaresetting);
					set _diag
					    _others;

						if Condinclusion=. then Condinclusion=1;
						if missing(excludesupply) then excludesupply = 'N';
						Code=compress(Code,'. ');
	    				CodeCat=upcase(CodeCat);

						if CodeCat="RX"          then output _ITGroupParamNDC;
						if CodeCat in("DX","PX") then output _ITGroupParamMeds;
						if CodeCat in("LB")      then output _ITGroupParamLabs;
				    run;

			    *Break out CaresettingPrincipal into Caresetting and Principal;
			    %ms_caresettingprincipal(InFile=_ITGroupParamMeds, 
			                             Var=CareSettingPrincipal, 
			                             OutFile=_ITGroupParamMeds);

			    *Extract milgrp-specific records;
			    %ms_loopmeds(datafile=dplocal.meds,
			                 lookfile=_ITGroupParamMeds, 
							 multiplecodecat=Y,
			                 outfile=_temp_PreMeds);

			   	%ms_extractdrugs(datafile=dplocal.drugs,
			                     datavar=NDC ,
			                     lookfile=_ITGroupParamNDC, 
			                     lookvar=code , 
			                     outfile=_temp_PreDrugs);

				%ms_extractdrugs(datafile=dplocal.drugscomb,
			                     datavar=NDC ,
			                     lookfile=_ITGroupParamNDC, 
			                     lookvar=code , 
			                     outfile=_temp_PreDrugsComb);

				%ms_extractlabs(datafile=dplocal.labextract,
		        	            lookfile=_ITGroupParamLabs, 
		            	        outfile=_temp_prelabs);	

				*restricting to mothers and linked infants;
				data _ITMeds(rename=DummyDate=ADate);
					if 0 then set PtsList&groupind.;
					declare hash pt (hashexp:16, dataset:"PtsList&groupind.");
					pt.definekey('PatId');
					pt.definedone();

					do until(eof1);
						set _temp_PreMeds end=eof1;
					    if pt.find()=0 then do;
							format DummyDate mmddyy10.;
							length DummyDate 4.;
							DummyDate=ADate;
							RxSup=1;
							RxAmt=1;
							drop ADate;
							output;
						end;
				    end;
				    stop;
				run;

			 	data _ITDrugs;
			        if 0 then set PtsList&groupind.;
			        declare hash pt1 (hashexp:16, dataset:"PtsList&groupind.");
			        pt1.definekey('PatId');
			        pt1.definedone();

			        do until(eof1);
			          set _temp_PreDrugs end=eof1;
			          if pt1.find()=0 then do;
					    if not missing(codesupply) then rxsup = codesupply;
						RxSupEndDt=Rxdate+RxSup-1; 	
						output;
					end;
			        end;
			        stop;
				run;

			 	data _ITDrugsComb;
			        if 0 then set PtsList&groupind.;
			        declare hash pt1 (hashexp:16, dataset:"PtsList&groupind.");
			        pt1.definekey('PatId');
			        pt1.definedone();

			        do until(eof1);
			          set _temp_PreDrugsComb end=eof1;
			          if pt1.find()=0 then do;
					    if not missing(codesupply) then rxsup = codesupply;
						RxSupEndDt=Rxdate+RxSup-1; 	
						output;
					end;
			        end;
			        stop;
				run;

				data _ITlabs(rename=DummyDate=ADate);
					if 0 then set PtsList&groupind.;
					declare hash pt (hashexp:16, dataset:"PtsList&groupind.");
					pt.definekey('PatId');
					pt.definedone();

					do until(eof1);
						set _temp_prelabs end=eof1;
					    if pt.find()=0 then do;
							format DummyDate mmddyy10.;
							length DummyDate 4.;
							DummyDate=ADate;
							RxSup=1;
							RxAmt=1;
							drop ADate;
							output;
						end;
				    end;
				    stop;
				run;

	 			*Make sure all claims overlap enrollment span and only 
			     keep eligible exposures (overlaping Enr_Start Enr_End);
			  	%ms_shaveoutside(reffile=_mi_ids_enr&groupind.,
			                     refstart=Enr_Start,
			                     refend=Enr_End,
			                     KeepPartBf=N,
			                     ToShaveFile=_ITDrugs,
			                     ToShaveStart=RxDate,
			                     ToShaveEnd=RxSupEndDt,
			                     outfile=_ITDrugs);

				%ISDATA(dataset=_ITDrugsComb);
				%if %eval(&NOBS.>=1) %then %do;
			    %ms_shaveoutside(reffile=_mi_ids_enr&groupind.,
			                     refstart=Enr_Start,
			                     refend=Enr_End,
			                     KeepPartBf=N,
			                     ToShaveFile=_ITDrugsComb,
			                     ToShaveStart=RxDate,
			                     ToShaveEnd=RxSupEndDt,
			                     outfile=_ITDrugsComb(drop=Enr_Start Enr_end));
			    %end;
			    
			    %ms_shaveoutside(reffile=_mi_ids_enr&groupind.,
			                     refstart=Enr_Start,
			                     refend=Enr_End,
			                     KeepPartBf=N,
			                     ToShaveFile=_ITMeds,
			                     ToShaveStart=ADate,
			                     ToShaveEnd=ADate,
			                     outfile=_ITMeds);
			    
			  	%ms_shaveoutside(reffile=_mi_ids_enr&groupind.,
			                   refstart=Enr_Start,
			                   refend=Enr_End,
			                   KeepPartBf=N,
			                   ToShaveFile=_ITLabs,
			                   ToShaveStart=ADate,
			                   ToShaveEnd=ADate,
			                   outfile=_ITLabs);   

			   %ISDATA(dataset=infolder.&STOCKPILINGFILE.);	
			    %IF %EVAL(&NOBS.>0) %THEN %DO;
			        data &STOCKPILINGFILE.;
			        set infolder.&STOCKPILINGFILE.;
			        where group="&milgrp.";
			        call symputx("SAMEDAY",strip(SameDay));
			        call symputx("SUPRANGE",strip(SupRange));
			        call symputx("AMTRANGE",strip(AmtRange));
			        call symputx("PERCENTDAYS",put(PercentDays,best.));
			        run;
			    %end;         

			    %MS_STOCKPILING(INFILE=_ITDrugs,	  		
			                    CLMDATE=RxDate,                            		
			                    CLMSUP=RxSup,                              		
			                    CLMAMT=RxAmt,	                           		
			        		    PROCFLAG=,                                 		
			        		    PERCENTDAYS=&PERCENTDAYS.,                          		
			        		    GROUPING=Group StockGroup T1_INDEX T2_INDEX T2_FUP T3_INDEX T3_FUP T4_INDEX T4_FUP T5_INDEX T6_INDEX CONC_FUP &InclExclVars.,                   		
			        		    SAMEDAY=&SAMEDAY.,                                 	
			        			SUPRANGE=&SUPRANGE.,									
			        			AMTRANGE=&AMTRANGE.,
			        		    ID=CODECAT,                                       		
			        		    OUTFILE=_ITDrugs,  		
			        			OUTFILEEXCL=_ITDrugsExcl
			                    );

			    data _ITDrugs;
			    set  _ITDrugs
			         _ITDrugsComb(in=a);
			    *ExpireDt does not exist in _ITDrugsComb;
			    if a then ExpireDt=Rxdate+RxSup-1; 	
			    run;

				*Need to reshave because Stockpiling may have pushed claims outside enrollment period;
			    %ISDATA(dataset=_ITDrugs);
				%IF %EVAL(&NOBS.>=1) %THEN %DO;
			        %ms_shaveoutside(reffile=_mi_ids_enr&groupind.,
			                       refstart=Enr_Start,
			                       refend=Enr_End,
			                       KeepPartBf=N,
			                       ToShaveFile=_ITDrugs,
			                       ToShaveStart=RxDate,
			                       ToShaveEnd=ExpireDt,
			                       outfile=_ITDrugs);
			    %END;	

		  		proc datasets library=work nowarn nolist;
			    	delete _diag _others _temp_:;
			    quit;

				*Create _InclExcl _FUPEvent and datasets;
				data _InclExcl
					 _FUPEvent(keep=PatId Adate ExpireDt Enr_Start codecat codetype code excludesupply stockgroup enctype pdx);
		
				    set _ITDrugs(in=a rename=RxDate=ADate rename=ndc=code)
				        _ITMeds(in=b)
				        _ITLabs(in=c);

					if b or c then do;
						if not missing(codesupply) then RxSup = CodeSupply;
						else RxSup = 1;
					    Expiredt=Adate+RxSup-1;
					    RXAmt=1;
						NumDispensing = 1;
				    end;

			        if T4_INDEX in('INC','EXC') then output _InclExcl;
		         	if T4_FUP in('DEF') then output _FUPEvent;
				run;


				/*****************************************/
				/* Evaluate inclusion/exclusion criteria */
				/*****************************************/

				*Are there any incl/excl conds to verify for this milgrp?;
				data _IT&inclusioncodes.;
					set &cohortcodes.;
				    where lowcase(group)="&milgrp.&groupind." and T4_INDEX in('INC','EXC');
				run;

				%isdata(dataset=_IT&inclusioncodes.);
				%if %eval(&nobs.>=1) %then %do;

					*Initialize list with all potential index dates;
				    data _pov3;
					    set _micohorts;
					    	MustKickOutHard=0;  *cumulative;
					        MustKickOutElig=0;  *cumulative;
						keep patid linkedid indexdt pregstartdt indexdt_exp pregenddt milID enr_start enr_end MustKickOut:;
					run;

					*Uniqueness of _InclExcl;
					proc sort nodupkey data=_InclExcl out=_InclExcl;
						by Patid Adate Expiredt CondInclusion cond subcond SubCondInclusion CondFrom CondTo codedays excludesupply codepop indexdate;
					run;

					*Split out inclusion/exclusion criteria into CodePop value;
					data _inclexcl_m
						 _inclexcl_i;
						set _inclexcl;

						if codepop in ('M', 'MI') then output _inclexcl_m;
						if codepop in ('I', 'MI') then output _inclexcl_i;
					run;

					*Extract cohort specific codes - mother and infant;
					proc sql noprint;
						create table _pov3claims as
				       	select distinct index.Patid,
						 				index.linkedid,
						               	index.indexdt,
										index.milID,
										index.pregstartdt,
										index.indexdt_exp,
										index.pregenddt,
						              	inc.CondInclusion,
						               	inc.cond,
						               	inc.Subcond,
						               	inc.SubCondinclusion,
									   	inc.codedays,
									  	inc.adate as incdate,
										upcase(inc.indexdate) as indexdate,
										inc.codepop
				        from _micohorts as index,
				            _inclexcl_m as inc
				        where index.Patid = inc.Patid and 

							/*INDEXDATE = PREGSTARTDT*/
							(	(upcase(inc.indexdate) = 'INDEXDT' &
								((inc.ExcludeSupply='N' and %MS_PeriodsOverlap(period1=COALESCE(index.pregstartdt+CondFrom,-999999) COALESCE(index.pregstartdt+CondTo,999999),                                             
					                     period2=Inc.ADate Inc.Expiredt)) |
								(inc.ExcludeSupply='Y' and %MS_PeriodsOverlap(period1=COALESCE(index.pregstartdt+CondFrom,-999999) COALESCE(index.pregstartdt+CondTo,999999),                                             
					                     period2=Inc.ADate)))) |

							/*INDEXDATE = INDEXDT_EXP*/
								(upcase(inc.indexdate) = 'INDEXDT_EXP' & 
								((inc.ExcludeSupply='N' and %MS_PeriodsOverlap(period1=COALESCE(index.INDEXDT_EXP+CondFrom,-999999) COALESCE(index.INDEXDT_EXP+CondTo,999999),                                             
					            		period2=Inc.ADate Inc.Expiredt)) |
							   	(inc.ExcludeSupply='Y' and %MS_PeriodsOverlap(period1=COALESCE(index.INDEXDT_EXP+CondFrom,-999999) COALESCE(index.INDEXDT_EXP+CondTo,999999),                                             
					                    period2=Inc.ADate)))) |

							/*INDEXDATE = PREGENDDT*/
								(upcase(inc.indexdate) = 'INDEXDT_DELIV' & 
								((inc.ExcludeSupply='N' and %MS_PeriodsOverlap(period1=COALESCE(index.PREGENDDT+CondFrom,-999999) COALESCE(index.PREGENDDT+CondTo,999999),                                             
				                        period2=Inc.ADate Inc.Expiredt)) |
						   		(inc.ExcludeSupply='Y' and %MS_PeriodsOverlap(period1=COALESCE(index.PREGENDDT+CondFrom,-999999) COALESCE(index.PREGENDDT+CondTo,999999),                                             
				                        period2=Inc.ADate)))) )

						union corr all

						/*Note: in reality infant claims should only look forward from PREGENDDT. Other options included for completeness*/

				        select distinct index.patid,
										index.linkedid,
						                index.indexdt,
										index.milID,
										index.pregstartdt,
										index.indexdt_exp,
										index.pregenddt,
						                inc.CondInclusion,
						                inc.cond,
						                inc.Subcond,
						                inc.SubCondinclusion,
										inc.codedays,
										inc.adate as incdate,
										upcase(inc.indexdate) as indexdate,
										inc.codepop
				  		from _micohorts as index,
				             _inclexcl_i as inc
				        where index.linkedid = inc.Patid and 
						
							/*INDEXDATE = PREGSTARTDT*/
							(	(upcase(inc.indexdate) = 'INDEXDT' &
								((inc.ExcludeSupply='N' and %MS_PeriodsOverlap(period1=COALESCE(index.pregstartdt+CondFrom,-999999) COALESCE(index.pregstartdt+CondTo,999999),                                             
					                     period2=Inc.ADate Inc.Expiredt)) |
								(inc.ExcludeSupply='Y' and %MS_PeriodsOverlap(period1=COALESCE(index.pregstartdt+CondFrom,-999999) COALESCE(index.pregstartdt+CondTo,999999),                                             
					                     period2=Inc.ADate)))) |

							/*INDEXDATE = INDEXDT_EXP*/
								(upcase(inc.indexdate) = 'INDEXDT_EXP' & 
								((inc.ExcludeSupply='N' and %MS_PeriodsOverlap(period1=COALESCE(index.INDEXDT_EXP+CondFrom,-999999) COALESCE(index.INDEXDT_EXP+CondTo,999999),                                             
					            		period2=Inc.ADate Inc.Expiredt)) |
							   	(inc.ExcludeSupply='Y' and %MS_PeriodsOverlap(period1=COALESCE(index.INDEXDT_EXP+CondFrom,-999999) COALESCE(index.INDEXDT_EXP+CondTo,999999),                                             
					                    period2=Inc.ADate)))) |

							/*INDEXDATE = PREGENDDT*/
								(upcase(inc.indexdate) = 'INDEXDT_DELIV' & 
								((inc.ExcludeSupply='N' and %MS_PeriodsOverlap(period1=COALESCE(index.PREGENDDT+CondFrom,-999999) COALESCE(index.PREGENDDT+CondTo,999999),                                             
				                        period2=Inc.ADate Inc.Expiredt)) |
						   		(inc.ExcludeSupply='Y' and %MS_PeriodsOverlap(period1=COALESCE(index.PREGENDDT+CondFrom,-999999) COALESCE(index.PREGENDDT+CondTo,999999),                                             
				                        period2=Inc.ADate)))) )
							;
					quit;

					*Determine how many conditions;
					%let MAXCOND=;
					proc sql noprint;
						select max(cond) into: maxcond
						from _IT&inclusioncodes.;
					quit;

					data _NULL_;
						call symputx("MAXCOND",&MAXCOND.);*For sum issue;
					run;
					%put &MAXCOND.;

						%do COND=1 %to &MAXCOND.;
							%let MAXSUBCOND=0;
							*Determine how many levels of subconditions needed;
							proc sql noprint;
								select max(subcond) into: maxsubcond
								from _IT&inclusioncodes. (where=(cond=&COND.));
							quit;

							data _NULL_;
								call symputx("MAXSUBCOND",&MAXSUBCOND.);*For sum issue;
							run;
							%put &MAXSUBCOND.;

							*Determine whether this COND is an exclusion or inclusion;
							proc sql noprint;
								select max(CondInclusion) into: inclusion
								from _IT&inclusioncodes. (where=(cond=&COND.)); 

								select min(CondFrom) into: MinimumCondFrom
								from _IT&inclusioncodes. (where=(cond=&COND.)); 
							quit;
							%put &INCLUSION. &MinimumCondFrom.;

							*Set data for this cond;
							data _pov3_&cond.;
								set _pov3(keep=milID indexdt pregstartdt indexdt_exp pregenddt enr_start enr_end);
				                ExclCond&COND.=0;
							run;

							%do SUBCOND=1 %to &MAXSUBCOND.;

								%let subinclusion=0;
								*Determine if SUBCOND is an exclusion or inclusion;
								proc sql noprint;

									select max(SubCondInclusion) into: SUBCONDINCLUSION
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

									select min(CondFrom) into: CondFrom
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

									select max(codedays) into: codedays
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

									select codepop into: codepop
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

									select upcase(indexdate) into: exclindexdate
									from _IT&inclusioncodes. (where=(cond=&COND. and subcond=&SUBCOND.)); 

								quit;
								%put &SUBCONDINCLUSION. &CondFrom. &codedays. &codepop. &exclindexdate.;

								*Only keep if codedays satisfied (number of ADates >= codedays);
								proc means data=_pov3claims noprint nway missing;
									class milID indexdt CondInclusion cond Subcond SubCondinclusion codedays codepop indexdate / missing;
									output out=_pov3Incl_tmp(drop=_: where=(freq>=codedays)) n=freq;
									where cond=&COND. and subcond=&SUBCOND.;
								run;

							    proc sort nodupkey data=_pov3Incl_tmp out=_pov3Incl_tmp(keep=milID Indexdt codepop indexdate);
							       	by milID Indexdt;
							      	where cond=&COND. and subcond=&SUBCOND.;
							    run;

								*Square;						
								data _pov3_&cond.;
									merge _pov3_&cond.(in=a)
									      _pov3Incl_tmp(in=b);
									by milID Indexdt;

									if a and b then SubCond_&COND._&SUBCOND.=1;
									else SubCond_&COND._&SUBCOND.=0;

									if &SUBCONDINCLUSION.=1 then do;
										if a and b then SubCond_&COND._&SUBCOND.=1;
										else SubCond_&COND._&SUBCOND.=0;
									end;
									*If the subcondition is met but it is a subexclusion, then means that condition not satisfied;
									else do;
										if a and b then SubCond_&COND._&SUBCOND.=0;
										else SubCond_&COND._&SUBCOND.=1;
									end;
								run;

					  			*Check elig for the subconditions exclusions - only for mother claims;
					          	*If the condition is a CondExclusion, then it will be checked again later with different assumptions;
									%if &SUBCONDINCLUSION.=0 and &inclusion.=1 %then %do;
										data _pov3_&cond.;
											set _pov3_&cond.;
											%if %index(&codepop., M) > 0 %then %do;
												if "&exclindexdate" = "INDEXDT" then evaldate = INDEXDT;
												if "&exclindexdate" = "INDEXDT_EXP" then evaldate = INDEXDT_EXP;
												if "&exclindexdate" = "INDEXDT_DELIV" then evaldate = pregenddt;

								          			if not(Enr_Start <= evaldate + COALESCE(&CondFrom.,0)) then do;*Open ended means -infty/infty;
								    					SubCond_&COND._&SUBCOND.=0;  
						               		 			ExclCond&COND.=1;
													end;
											%end;
										run;
									%end;
							%end; *loop subcond;
								
							*If all subconditions are satisfied, then condition is satisfied (at this point);
							data _pov3_&cond.;
								set _pov3_&cond.;
								Cond&cond.=sum(of SubCond_&COND._1-SubCond_&COND._&MAXSUBCOND.)=&MAXSUBCOND.;
							run;
							
							%let condFrom2=.;
							data _null_;
							set _IT&inclusioncodes. (where=(cond=&COND.));
							if CondInclusion=0 then do;
								call symputx('CondFrom2',"&MinimumCondFrom.");
							end;
							run;
							%let codepop2 = 0;
							%let exclindex2 = ;
							data _null_;
							set _IT&inclusioncodes. (where=(cond=&COND. and codepop in ('M', 'MI')));
								if _n_ = 1 then do;
									call symputx('codepop2',"1");
									call symputx('exclindex2', upcase(indexdate));
								end;
							run;
							%put &CondFrom. &CondFrom2. &codepop2.;

						    *Manage if an exclusion - if you have one hard exclusion or not enough elig, you are out;
							data _pov3_&cond.;
								set _pov3_&cond.;

								if &INCLUSION.=0 then do;
								    *if the milID had cond&cond.=1 then can never be included;
							        if Cond&cond.=1 then MustKickOutHard_&cond.=1;
							    	    else MustKickOutHard_&cond.=0;
							        *flip the dummy;
							        Cond&cond.=Cond&cond.=0;
							  		*Because we can have CondInclusion=0 and SubInclusion=1, we still need to consider if enough elig in this case;
							         MustKickOutElig_&cond.=0;
									 %if "&codepop2" = "1" %then %do;
										if "&exclindex2" = "INDEXDT" then evaldate = INDEXDT;
										if "&exclindex2" = "INDEXDT_EXP" then evaldate = INDEXDT_EXP;
										if "&exclindex2" = "INDEXDT_DELIV" then evaldate = pregenddt;

							         if not(Enr_Start <= evaldate + COALESCE(&CondFrom2.,0)) then do;
							            MustKickOutElig_&cond.=1;
							           ExclCond&COND.=1;  *Open ended means -infty/infty;
							            Cond&cond.=0;
							          end;
									%end;
									  end;
									*Create a dummy to remember if this condition was an inclusion or and exlusion;
									InclCond&COND.=&INCLUSION.;
							run;

							*Put all together;
							data _pov3;
								merge _pov3(in=a)
								      _pov3_&cond.(in=b keep=milID indexdate Cond: SubCond: Excl: InclCond: MustKickOut:);
								by milID;
							    if MustKickOutHard_&cond.=1 then MustKickOutHard=1;
							    if MustKickOutElig_&cond.=1 then MustKickOutElig=1;
								if a;
							run;
						%end;*loop cond;

						*Clean up;
						data _pov3(where=(anycond=1));															
							set _pov3;
						    array _cond(*) Cond1-cond&maxcond.;
						    array _InclCond(*) InclCond1-InclCond&maxcond.;

						    *at least one inclusion criterion;
						    %if &maxcond. >1 %then %do;
								inclusion=max(of InclCond1-InclCond&maxcond.);
							%end;
							%else %do;
						       	inclusion=InclCond1;
							%end;

						    *initialize AnyCond;
						    if inclusion ne 1 then AnyCond=1;
							else do;
						    	AnyCond=0;
						        *Find those with at least on inclusion met;
						    	do i=1 to &maxcond.;
						        	if _cond(i)=1 and _InclCond(i)=1 then AnyCond=1;
						    	end;	
						    end;
						 
						    *kick out any index not meeting the elig criteria for at least one exclusion
						     or that has at least one hard exclusion (claim).;
						    if MustKickOutElig or MustKickOuthard then AnyCond=0;
						    drop inclusion;
						 run;

						data _micohorts&groupind.;
						  	merge _micohorts(where=(group="&milgrp.&groupind.") in=a)
								  _pov3(in=b keep=milID);
								by milID;
								if a and b;
						run;

				%end;  /*INCLUSIONCODES*/
				%else %do;
					data _micohorts&groupind.;
						set _micohorts(where=(group="&milgrp.&groupind."));
					run;
				%end;

				/************************************/
				/*Restrict to 1st episode per group */
				/************************************/
				proc sort data=_micohorts&groupind.;
					by patid indexdt;
				run;

				data _micohorts&groupind.;
					set _micohorts&groupind.;
					by patid indexdt;
					if first.patid;
				run;

				/*********************/
				/* Evaluate outcomes */
				/*********************/

				*Select first event date in outcome observation window;

				*Are there any FUP codes for this milgrp?;
				data _ITFUPoutcomes;
					set &cohortcodes.;
				    where lowcase(group)="&milgrp.&groupind." and T4_FUP in('DEF');
				run;

				%isdata(dataset=_ITFUPoutcomes);
				%if %eval(&nobs.>=1) %then %do;

					*Extract cohort specific codes - mother and infant;
					%if %index(&outcomepop,M) > 0 %then %do;
						proc sql noprint;
							create table _episodeswitheventsM(where=(InGap=1)) as
					        select distinct index.Patid,
											index.linkedid,
					                        index.IndexDt,
											index.milID,
					                        inc.Adate as EventDt,
											inc.Code,
					                        %MS_PeriodsOverlap(period1=index.outcomestart index.outcomeend,                                             
					                                           period2=Inc.ADate Inc.ExpireDt) as InGap
					        from _micohorts&groupind. as index, 
					             _FUPEvent as Inc
					        where index.Patid = Inc.Patid;
						quit;
					%end;
					%if %index(&outcomepop,I) > 0 %then %do;
						proc sql noprint;
							create table _episodeswitheventsI(where=(InGap=1)) as
					        select distinct index.Patid,
											index.linkedid,
					                        index.IndexDt,
											index.milID,
					                        inc.Adate as EventDt,
											inc.Code,
					                        %MS_PeriodsOverlap(period1=index.outcomestart index.outcomeend,                                             
					                                           period2=Inc.ADate Inc.ExpireDt) as InGap
					        from _micohorts&groupind. as index, 
					             _FUPEvent as Inc
					        where index.linkedid = Inc.Patid;
						quit;
					%end;

					*Select 1st event date in outcome window and merge with master episode list;
					data _episodeswithevents;
						set 
						%if %index(&outcomepop,M) > 0 %then %do;
							_episodeswitheventsM
						%end;
						%if %index(&outcomepop,I) > 0 %then %do;
							_episodeswitheventsI
						%end;	
						;
					run;

			        proc means data=_episodeswithevents nway noprint;
			        	var InGap EventDt;
			        	class milID IndexDt;
			        	output out=_episodeswithevents(drop=_:) sum(InGap)=NumEvents min(EventDt)=FEventDt/KEEPLEN;
			        run;
			 
					data _micohorts&groupind.;
						merge _micohorts&groupind.(in=a)
					          _EPisodesWithEvents(in=b);
					    by milID IndexDt;
					    if a;
					run;
				%end;
				%else %do;
					data _micohorts&groupind.;
						set _micohorts&groupind.;
							length feventdt 4.;
							NumEvents = .;
							FEventDt = .;
					run;
				%end;
		%mend;
		%mi_pov3_5(groupind=_ref);
		%mi_pov3_5(groupind=_eoi);

		*Combine _eoi and _ref cohorts;
		data _micohorts;
			set _micohorts_eoi _micohorts_ref;

			*drop moi metrics - need to compute per patient and not per MOI episode for baseline/table 1;
			drop usepre anyt: onlyt: sum: gestwk;
		run;

		/****************************************************************/
		/* Compute MOI metrics for final cohorts - for baseline/table 1 */
		/****************************************************************/

		proc sql noprint;
			create table _alldisp as
			select pts.patid,
				   pts.pregstartdt,
				   moi.moiname,
				   moi.gestwk,
				   moi.sumadjcntpre,
				   moi.sumadjcntanyt1,
				   moi.sumadjcntanyt2,
				   moi.sumadjcntanyt3,
				   moi.anyt1,
				   moi.anyt2,
				   moi.anyt3,
				   moi.usepre
			from _micohorts as pts
			left join dplocal.&runid._concepi_mstr_preg_tri(where=(lowcase(group)="&groupname")) as moi
			on pts.patid = moi.patid and pts.pregstartdt = moi.indexdt and pts.moiname = moi.moiname;
		quit;

		*summarize;
		proc means data=_alldisp nway noprint missing;
			var gestwk UsePre AnyT1 AnyT2 AnyT3 SumAdjCntPre SumAdjCntAnyT1 SumAdjCntAnyT2 SumAdjCntAnyT3;
	   		class PatId moiname pregstartdt ; 	
			output out=_moi_by_del (drop=_:) min(gestwk) = ga_first
											 max(usepre)=exp_pre
											 max(anyt1)=exp_t1
											 max(anyt2)=exp_t2
											 max(anyt3)=exp_t3
											 sum(SumAdjCntPre)=adjusteddisp_pre
											 sum(SumAdjCntAnyT1)=adjusteddisp_t1
											 sum(SumAdjCntAnyT2)=adjusteddisp_t2
											 sum(SumAdjCntAnyT3)=adjusteddisp_t3;			
		run;

		proc sql noprint;
			create table MICohortMstList as
			select pts.*,
				case when moi.ga_first > . then moi.ga_first
				else 0
				end as ga_first,
				case when moi.exp_pre > . then moi.exp_pre
				else 0
				end as exp_pre,
				case when moi.exp_t1 > . then moi.exp_t1
				else 0
				end as exp_t1,
				case when moi.exp_t2 > . then moi.exp_t2
				else 0
				end as exp_t2,
				case when moi.exp_t3 > . then moi.exp_t3
				else 0
				end as exp_t3,
				case when moi.adjusteddisp_pre > . then moi.adjusteddisp_pre
				else 0
				end as adjusteddisp_pre,
				case when moi.adjusteddisp_t1 > . then moi.adjusteddisp_t1
				else 0
				end as adjusteddisp_t1,
				case when moi.adjusteddisp_t2 > . then moi.adjusteddisp_t2
				else 0
				end as adjusteddisp_t2,
				case when moi.adjusteddisp_t3 > . then moi.adjusteddisp_t3
				else 0
				end as adjusteddisp_t3
			from _micohorts as pts
			left join _moi_by_del as moi
			on pts.patid = moi.patid and pts.pregstartdt = moi.pregstartdt and pts.moiname = moi.moiname;
		quit;

		*Save to DPlocal;
		%if %eval(&a.=1) %then %do;
  			data DPLocal.&RUNID._mstr_mi;
		        set MICohortMstList;
		    run;
		%end;
		%else %do;
			proc append base=DPLocal.&RUNID._mstr_mi
			     		data=MICohortMstList force;
		%end;	

		/*Output run time*/
		%ms_stoptimer(timestart=micohortruntime);
		%ms_outputruntimes(timevar=micohortruntime, step=%str(Create MI cohorts), group=&milgrp., monitoringperiod=);

		%end;
	%end;

	*Sort for ADS;
	proc sort data=DPLocal.&RUNID._mstr_mi; 
		by group patid indexdt; 
	run;

%put NOTE: ********END OF MACRO: ms_createmicohorts v1.2********;

%mend ms_createmicohorts;
