****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createnopreggroup.sas  
*
* Created (mm/dd/yyyy): 08/24/2015
* Last modified: 06/30/2019
* Version: 3.2
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create control groups.                                        
*   
*  Program inputs:  
*	- a dataset containing enrolment episodes
*	- a dataset containing inclusion/exclusion codes
*	- a dataset containing defined group pregnancy episodes
*	- a dataset containing the patient master list
*             	                        
*  Program outputs:                                                                                                                                       
*	- datasets with a list of control patients and demo data
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    2.0	  25MAY2016  SL			Implement enhanced HOI
*
*    2.1	  23MAR2017  DM			Added variable in control group creation
*
*	 2.2	  05/26/17 	 AP			Added EnrDaysAft criteria
*
*	 2.3	  07/20/17	 AP			Added demographic info for baseline table, 
*									changed variable name DelYear to Year
*
*	 2.4	  10/15/17	 AP			Bug fix to keep demographic vars, change age to control grp (QRP-449, QRP-452)

*	 2.5	  10/10/17	 AP			Added geographic location information
*
*	 2.6	  12/21/17	 AP			Added CODEDAYS to inclusion/exclusion criteria (QRP-455)
*
*	 2.7	  01/02/18	 AP			Removed post-index enrollment (QRP-496)
*
*	 2.8	  04/22/18	 AP			Added DelNum and GestAge of treat to ctrl dataset
*
*	 2.9	  05/15/18	 AP			Added ExcludeSupply parameter, changed EnrDaysAft to EnrDaysAftInd
*									Made control group creation optional
*									Added AgeGroupNum, changed group length to $40
*
*	 3.0	  10/20/18	 AP			Changed macro name to ms_createnopreggroup
*
*	 3.1	  11/12/18	 AP			Directed large datasets to DPLOCAL
*
*	 3.2	  06/30/19	 AP			Added IndexLook and Month variables of treat group
*
***************************************************************************************************;

%macro ms_createnopreggroup();

	%put =====> MACRO CALLED: ms_createnopreggroup v3.1;

	%if "&t4_nopreggroup." = "Y" %then %do; /*Only create non-pregnant group if requested*/

		* Create basic control population;
		%ms_agestrat(infile=dplocal.enr_&enrollmentnum.,
		             outfile=dplocal._controlpop,
		             startdt=birth_date,
		             enddt=Enr_End,
		             timestrat=&agestrat.);

		* Apply age group and enrollment duration requirements;
		data dplocal._controlpop (where=(Overlap=1 and Enr_end-Enr_start >= &EnrDays.+&enrDaysAftInd.));
		set dplocal._controlpop;
		Overlap= %MS_PeriodsOverlap(period1=Enr_Start Enr_End,period2=MinAgeDate MaxAgeDate);
		run; 

		* Identify enrollment episodes where members also appeared in the treatment group;
		proc sort data= dplocal._controlpop;
		by PatID Enr_Start Enr_End;
		run;

		proc sort nodupkey data= _GroupEpis out=_UniqueGroupEpis;
		by patid Episode;
		run;

		proc sort data= _UniqueGroupEpis out=_UniqueGroupEpis;
		by PatID Enr_Start Enr_End;
		run;

		data dplocal._controlpopepistreat dplocal._controlpopepisnotreat;

		merge dplocal._controlpop (in=a) _UniqueGroupEpis;
		by PatID Enr_Start Enr_End;
		if a;
		if Enr_Start <= ADate <= Enr_End then output dplocal._ControlPopEpisTreat;	*Enrollment episodes where treatment occured;
		else output dplocal._ControlPopEpisNoTreat;									*Enrollment episodes without treatment;
		keep PatID Enr_Start Enr_End Birth_Date;
		run;

		data _UniqueGroupEpis;	
		set _UniqueGroupEpis;	
		drop Enr_Start Enr_End;	*Must be removed before shave inside macro call below;
		run;

		*Format dates to avoid w.a.r.n.i.n.g - used to be MMDDYY10.;
		data dplocal._controlpopepistreat;
		set dplocal._controlpopepistreat;
		format enr_start enr_end date9.;
		run;

		* Remove treatment episodes segments from the control population enrollment episodes;
		%ms_shaveinside(reffile=dplocal._controlpopepistreat,
			             refstart=Enr_Start,
			             refend=Enr_End,                         
			             ToShaveFile=_UniqueGroupEpis,
			             ToShaveStart=EpisodeStartDt,
			             ToShaveEnd=EpisodeEndDt,
			             outfile=dplocal._controlpopepistreat);

		* Get demographic info back;
		proc sql noprint undo_policy=none;
		create table dplocal._controlpopepistreat as
		select cont1.*,
			   cont2.birth_date,
			   cont2.sex,
			   cont2.race,
			   cont2.hispanic,
			   cont2.zip,
			   cont2.zip_date
		from dplocal._controlpopepistreat as cont1, dplocal._controlpop as cont2
		where cont1.patid=cont2.patid;
		quit;

		proc sql noprint undo_policy=none;
		create table dplocal._controlpopepisnotreat as
		select cont1.*,
			   cont2.sex,
			   cont2.race,
			   cont2.hispanic,
			   cont2.zip,
			   cont2.zip_date
		from dplocal._controlpopepisnotreat as cont1, dplocal._controlpop as cont2
		where cont1.patid=cont2.patid;
		quit;
			
		data dplocal._controlpopulation;
		set dplocal._controlpopepistreat 
		    dplocal._controlpopepisnotreat;
		run;

		proc sort nodupkey data= dplocal._controlpopulation;
		by PatID Enr_Start Enr_End;
		where Enr_end-Enr_start >= &enrdays.+&enrdaysaftind.; *Reapply the criterion since removing treatment episodes may have created too short segments;
		run;

		/* There are some discrepancies with the original pregnancy code caused by the fact that the shaveinside macro does 
		* not only shave the enrollment episode where the delivery occurs but shaves also previous enrollment episodes
		* For example, enrollment episode 1 (01AUG2004 - 31MAY2006) and episode 2 (01AUG2006 - 31JAN2007), delivery 19NOV2006
		* ShaveInside will truncate episode 1 at 19FEB2006 even if the delivery occured in episode 2
		*/ 
		data dplocal._controlpopulation;
		set dplocal._controlpopulation;
		idc = _N_;
		MinAge = floor((intck('month',Birth_Date,Enr_Start+&enrdays.) - (day(Enr_Start+&enrdays.) < day(Birth_Date))) / 12);
		MaxAge = floor((intck('month',Birth_Date,Enr_End-&enrdaysaftind.) - (day(Enr_End-&enrdaysaftind.) < day(Birth_Date))) / 12);
		*Dummies used when there is no PEC;
		CStart=-99999;
		CEnd=999999;*If no inclusion exclusion, leave as is;
		run;


	*START code for complex HOI; 

		*Create a copy to have control info in the case of exclusion only;
		data dplocal._controlpopulation_copy;
		set dplocal._controlpopulation;
		run;

		* Extract highest CONDFROM value among exclusions if any: apply if GROUP has incl-excl codes, else do not change _controlpopulation before matching;
	    %let MaxExclCondFrom=0;
	    %macro wrapper;
	    	%ISDATA(dataset=&INCLUSIONCODES.);
	    	%IF %EVAL(&NOBS.>=1) %THEN %DO;
				data _IT&INCLUSIONCODES.;
				set &INCLUSIONCODES.;
				where group="&ITGROUP.";
				run; 

		    	%ISDATA(dataset=_IT&INCLUSIONCODES.);
				%IF %EVAL(&NOBS.>=1) %THEN %DO;
		    		proc sql noprint;
		    		select min(condfrom) into :MaxExclCondFrom 						
		    		from &INCLUSIONCODES.(where=(group="&ITGROUP.")) 
		    		where Condinclusion=0;
		    		quit;

				    data dplocal._controlpopulation;
				    set dplocal._controlpopulation;
				    format  Enr_Start Enr_End DenomEnrStart DenomEnrEnd mmddyy10.;

					*Note: in pregnancy, enrdays is from delivery date, not episode start date and it is already applied above;	
				    MaxLookBack=Max(0,-&MaxExclCondFrom.);	

				    * Resctrict eligibility to Study period (minus maximum of required lookbacks);
				    DenomEnrStart=Enr_Start;
				    * Initially truncate start eligibility enrollment start by max of lookback periods;
				    DenomEnrStart=DenomEnrStart + MaxLookBack; 									
				    DenomEnrEnd=Enr_End;
				    if DenomEnrEnd >= DenomEnrStart;

					*Create MaxLookBack macro variable;
					call symputx('MaxLookBack',MaxLookBack);

					*If there is inclusion or exclusion use DenomEnr instead of CStart CEnd;
					drop CStart CEnd;
				    run;
					%put &MaxLookBack.;

			    %END;
			    %put &MaxExclCondFrom.;

			%END;
		   %mend wrapper;
	    %wrapper;


		/*---------------------------------------------------*/
		/* Inclusion required (POV3 Incl) for eligibility	 */
		/*---------------------------------------------------*/  
		proc datasets library=work nowarn nolist;
		delete _control_stack:;
		quit;

		%macro wrapper;
			%ISDATA(dataset=&INCLUSIONCODES.);
			%IF %EVAL(&NOBS.>=1) %THEN %DO;

				data _IT&INCLUSIONCODES.;
				set &INCLUSIONCODES.;
				where group="&ITGROUP.";
				run; 

				*For EACH condition, shave periods for all subconditions;
				%ISDATA(dataset=_IT&INCLUSIONCODES.);
				%IF %EVAL(&NOBS.>=1) %THEN %DO;
					%let MAXCOND=;
					proc sql noprint;
					select max(cond) into: maxcond
					from _IT&INCLUSIONCODES.;
					quit;
					%put &MAXCOND.;

					%DO COND=1 %TO &MAXCOND.;
						*Determine how many levels of subconditions needed;
						%let MAXSUBCOND=;
						proc sql noprint;
						select max(subcond) into: maxsubcond
						from _IT&INCLUSIONCODES. (where=(COND=&COND.));
						quit;
						*Determine if condition is an inclusion;
						proc sql noprint;
						select max(CondInclusion) into: inclusion
						from _IT&INCLUSIONCODES. (where=(COND=&COND.));
						quit;
						%put &MAXSUBCOND. &INCLUSION.;

						*Determine if there is subexclusion in condition in order to calculate new maxlookback since one above is for CondInclusion=0;
						%let SUBEXCL=0;
			            data _null_;
			            set _IT&INCLUSIONCODES.(where=(COND=&COND.));
			            if subCondinclusion=0 then call symputx("SUBEXCL",1);
			            run;
			            %put SUBEXCL = &SUBEXCL.;

						*If there is subexclusion, determine new maxlookback for that specific condition before shaving;
						%IF &SUBEXCL.=1 %THEN %DO;
						    %let NewMaxExclCondFrom=0;
						    proc sql noprint;
						    select min(condfrom) into :NewMaxExclCondFrom 
						    from _IT&INCLUSIONCODES. 									
						    where COND=&COND. and SubCondInclusion=0;
						    quit;
						    %put &NewMaxExclCondFrom.;

						    data _ControlPopulation_&cond. (rename=(newmaxlookback=MaxLookBack));
						    set dplocal._controlpopulation;
							*If the subcond condfrom is greater in absolute value than the maxlookback value, then readjust denom period;
						    NewMaxLookBack=Max(0,&Maxlookback.,-&NewMaxExclCondFrom.);
						    * Resctrict eligibility to Study period (minus maximum of required lookbacks);
						    DenomEnrStart=Enr_Start;
						    * Initially truncate start eligibility enrollment start by max of lookback periods;
						    DenomEnrStart=DenomEnrStart + NewMaxLookBack;
						    DenomEnrEnd=Enr_End;
						    if DenomEnrEnd >= DenomEnrStart;
							  drop maxlookback;
						    run;
						%END;*If subexcl=1;
						%IF &SUBEXCL.^=1 %THEN %DO;
							data _ControlPopulation_&cond.;
							set dplocal._controlpopulation;
							run;
						%END;

						%DO SUBCOND=1 %TO &MAXSUBCOND.;
							*Determine if subcond is an inclusion or exclusion;
							%let SUBINCLUSION=0;
		 					proc sql noprint;
							select max(SubCondInclusion) into: subinclusion
							from _IT&INCLUSIONCODES. (where=(COND=&COND. and SUBCOND=&SUBCOND.));
							quit;
							%put &SUBINCLUSION.;

							*Merge time periods for each subcondition and shave for each;
							%IF &SUBINCLUSION. = 1 %THEN %DO;

								proc sort data=_InclExcl out=_EligIncl_&subcond.;
									by patid adate;
									where COND=&COND. and subcond=&SUBCOND.; 
								run;

								data _EligIncl_&subcond.;
							    set _EligIncl_&subcond.;
								    format EligStart EligEnd date9.;
								    EligStart=ADate-coalesce(CondTo,99999);
									if excludesupply = 'N' then do;
								    	EligEnd=Expiredt-coalesce(CondFrom,-99999);
									end;
									if excludesupply = 'Y' then do;
								    	EligEnd=Adate-coalesce(CondFrom,-99999);
									end;
								    if EligStart<=EligEnd;
								    id = _n_;
							  	  keep PatId ADate EligStart EligEnd codedays id;
							    run;

								*only keep time periods that meet codedays criteria;
								%macro evaluate_codedays_inc();
									%let codedays = 1;
									data _null_;
										set _EligIncl_&subcond.;
										if _n_ = 1 then do;
											call symputx('codedays', codedays);
										end;
									run;

									%if %eval(&codedays.) = 1 %then %do;
										data _overlap_spans_&subcond.;
											set _EligIncl_&subcond.;
										run;
									%end;
									%else %do;
										proc sql noprint;
											create table _overlap_spans_&subcond.(where=(eligstart <=eligend) rename=overlap_start=eligstart rename=overlap_end=eligend) as
											select distinct
												a1.patid,
												max(a1.eligstart %do ref = 2 %to &codedays.; ,a&ref..eligstart %end;) as overlap_start format date9.,
												min(a1.eligEnd %do ref = 2 %to &codedays.; ,a&ref..eligEnd %end;) as overlap_end format date9.
											from _EligIncl_&subcond. as a1
												%do ref = 2 %to &codedays.;
												, _EligIncl_&subcond. as a&ref.
												%end;
											where a1.Patid %do ref = 2 %to &codedays.; = a&ref..PatId %end; and (a1.id %do ref = 2 %to &codedays.; < a&ref..id %end;); 
										quit;
									%end;
								%mend;
								%evaluate_codedays_inc();

								%MergeTimePeriods(dataset=_overlap_spans_&subcond.,
								                  DateStart=EligStart,
								                  DateEnd=EligEnd);	

								%ms_shaveoutside(reffile=_overlap_spans_&subcond.,
								              	 refstart=EligStart,
								               	 refend=EligEnd,  
											   	       KeepPartBf=Y,
								               	 ToShaveFile=_controlpopulation_&cond.,
								               	 ToShaveStart=DenomEnrStart,
								               	 ToShaveEnd=DenomEnrEnd,
								               	 outfile=_controlpopulation_&cond.);

								*Drop refstart and refend since _Denom&Group can be looped;
								data _controlpopulation_&cond.;
								set _controlpopulation_&cond.(drop=eligstart eligend);
								run;
							%END;*If subinclusion=1;
							%IF &SUBINCLUSION. = 0 %THEN %DO;
								proc sort data=_InclExcl out=_UneligExcl_&subcond.;
									by patid adate;
									where COND=&COND. and subcond=&SUBCOND.; 
								run;

						        data _UneligExcl_&subcond.;
							        set _UneligExcl_;
							        format UneligStart UneligEnd MMDDYY10.;
							        UneligStart=ADate-coalesce(CondTo,99999);
									if excludesupply = 'N' then do;
							        	UneligEnd=Expiredt-coalesce(CondFrom,-99999);
									end;
									if excludesupply = 'Y' then do;
							        	UneligEnd=Adate-coalesce(CondFrom,-99999);
									end;
							        if UneligStart<=UneligEnd;
									id = _n_;
							        keep PatId ADate UneligStart UneligEnd codedays id;
						        run;

								*only keep time periods that meet codedays criteria;
								%macro evaluate_codedays_exl();
									%let codedays = 1;
									data _null_;
										set _UneligExcl_&subcond.;
										if _n_ = 1 then do;
											call symputx('codedays', codedays);
										end;
									run;

									%if %eval(&codedays.) = 1 %then %do;
										data _overlap_spans_&subcond.;
											set _UneligExcl_&subcond.;
										run;
									%end;
									%else %do;
										proc sql noprint;
											create table _overlap_spans_&subcond.(where=(Uneligstart <=Uneligend) rename=overlap_start=Uneligstart rename=overlap_end=Uneligend) as
											select distinct
												a1.patid,
												max(a1.UneligStart %do ref = 2 %to &codedays.; ,a&ref..Uneligstart %end;) as overlap_start format date9.,
												min(a1.UneligEnd %do ref = 2 %to &codedays.; ,a&ref..UneligEnd %end;) as overlap_end format date9.
											from _UneligExcl_&subcond. as a1
												%do ref = 2 %to &codedays.;
												, _UneligExcl_&subcond. as a&ref.
												%end;
											where a1.Patid %do ref = 2 %to &codedays.; = a&ref..PatId %end; and (a1.id %do ref = 2 %to &codedays.; < a&ref..id %end;); 
										quit;
									%end;
								%mend;
								%evaluate_codedays_inc();


								%MergeTimePeriods(dataset=_overlap_spans_&subcond.,
								                  DateStart=UneligStart,
								                  DateEnd=UneligEnd);

								*Verify if there is data or else e.r.r.o.r when shaving;
								%ISDATA(dataset=_UneligExcl_&subcond.);
								%IF %EVAL(&NOBS.>=1) %THEN %DO;
								    %ms_shaveinside(reffile=_controlpopulation_&cond.,
								                   	refstart=DenomEnrStart,
								                    refend=DenomEnrEnd,  
								                    ToShaveFile=_overlap_spans_&subcond.,
								                    ToShaveStart=UneligStart,
								                    ToShaveEnd=UneligEnd,
								                    outfile=_controlpopulation_&cond.);
								%END;*If there is data;
							%END;*If subinclusion=0;
						%END;*loop on subcond;

						%put &INCLUSION.;
						%IF &INCLUSION.=1 %THEN %DO;
							proc append base=_control_STACK_INC
								data=_controlpopulation_&cond.(keep=patid denomenrstart denomenrend) force;
								run;
						%END;

						%IF &INCLUSION.=0 %THEN %DO;
							proc append base=_control_STACK_EXCL
								data=_controlpopulation_&cond.(keep=patid denomenrstart denomenrend) force;
							    run;
						%END;

					%END;*End Loop Cond;

					*Output dataset for inclusion=1 and for exclusions=0 and then (a) shaveoutside (b) shaveinside denom; 
					*For inclusions, need to merge periods;

					*First shaveoutside denom file but need to merge inclusion periods together;
					%IF %SYSFUNC(exist(_CONTROL_STACK_INC))=1 %THEN %DO;
						%MergeTimePeriods(dataset=_CONTROL_STACK_INC,
								          DateStart=DenomEnrStart,
								          DateEnd=DenomEnrEnd);


				        *_Denom&Group._STACK_INC includes all time periods where the potential index dates will meet at least one of the cond inclusions;
						data _CONTROL_STACK_INC;
						set _CONTROL_STACK_INC;
						rename 
						DenomEnrStart=DTSTART
						DenomEnrEnd=DTEND
						;
						run;

						%ms_shaveoutside(reffile=_CONTROL_STACK_INC,
									   	 refstart=DTSTART,
								       	 refend=DTEND,  
									   	 /*id=EligEpisode,*/ 
									   	 KeepPartBf=Y,
								       	 ToShaveFile=dplocal._controlpopulation,
								       	 ToShaveStart=DenomEnrStart,
								       	 ToShaveEnd=DenomEnrEnd,
								       	 outfile=dplocal._controlpopulation);	

					%END;

					%IF %SYSFUNC(exist(_CONTROL_STACK_EXCL))=1 %THEN %DO;

				        *merge all periods of ineligibilities;
						%MergeTimePeriods(dataset=_CONTROL_STACK_EXCL,
								          DateStart=DenomEnrStart,
								          DateEnd=DenomEnrEnd);

					      *_Denom&Group._STACK_EXCL includes all time periods where the potential index dates will meet at least one of the cond exclusions;
						data _CONTROL_STACK_EXCL;
						set _CONTROL_STACK_EXCL;
						rename 
						DenomEnrStart=DTSTART
						DenomEnrEnd=DTEND
						;
						run;

						*Defensive coding if exclusion codes only;
						data dplocal._controlpopulation;
						format DenomEnrStart DenomEnrEnd Date9.;
						set dplocal._controlpopulation;
						run;

				        *remove all these periods from the ELIG;
						%ms_shaveinside(reffile=dplocal._controlpopulation,
							           	refstart=DenomEnrStart,
							            refend=DenomEnrEnd,  
										/*id=ENR_START ENR_END,*/				    
							            ToShaveFile=_CONTROL_STACK_EXCL,
							            ToShaveStart=DTSTART,
							            ToShaveEnd=DTEND,
							            outfile=dplocal._controlpopulation);
					%END;

					proc sort data=dplocal._controlpopulation out=dplocal._controlpopulation nodupkey;						
					by patid denomenrstart denomenrend;
					run;

					*Need to rename if inside the inclusion code loop;
					data dplocal._controlpopulation;
					set dplocal._controlpopulation ;
					rename 
					DenomEnrStart=CStart
					DenomEnrEnd=CEnd;
					run;

					*If there was exclusion, other variables lost in shaveinside so merge back in;
					proc contents data=dplocal._controlpopulation out=_name (where=(name="ENR_START"));
					run;
					%ISDATA(dataset=_name);
					%IF %EVAL(&NOBS.=0) %THEN %DO;
						*Match treatment with control;
						proc sql noprint;
						create table _tmp_controlpopulation as
						select cont.Patid,
							cont.CStart,
							cont.CEnd,
							info.enr_start,
							info.enr_end,
							info.birth_date,
							info.minage,
							info.maxage,
							info.sex,
							info.race,
							info.hispanic,
							info.zip,
				   			info.zip_date
				      	from dplocal._controlpopulation as cont,
						     dplocal._controlpopulation_copy as info
						where info.enr_start <= cont.Cstart<= info.enr_end and info.patid=cont.patid
						order by patid; 
						quit;

						data dplocal._controlpopulation;
						set _tmp_controlpopulation;
						run;

						proc datasets nowarn noprint nolist lib=work;
							delete _tmp_controlpopulation;
						quit;
					%END;

				%END;*If there are inclusion codes for GROUP;
			%END;*If there are inclusion codes;

	%mend wrapper;
	%wrapper;


	/*-----------------------------------------------------*/
	/* Make sure the number of enrollment days remaining   */
	/* after each potential elig days is sufficient		   */
	/*-----------------------------------------------------*/
		proc sort data=dplocal._controlpopulation; 
			by patid;
		run;

	    data dplocal._controlpopulation;
		retain patid birth_date Enr_Start Enr_End CStart CEnd IDC minage maxage;
	    set dplocal._controlpopulation;
	    by PatId;
	    format AdjustedEnrEnd date9.;

	    * Adjust eligibility end date;
	    AdjustedEnrEnd = Enr_End;
	    CEnd = min(CEnd,AdjustedEnrEnd);
		IDC=_N_;
		drop adjustedenrend;
	    
		MinAge = floor((intck('month',Birth_Date,Enr_Start+&enrdays.) - (day(Enr_Start+&enrdays.) < day(Birth_Date))) / 12);
		MaxAge = floor((intck('month',Birth_Date,Enr_End-&enrdaysaftind.) - (day(Enr_End-&enrdaysaftind.) < day(Birth_Date))) / 12);
		if (Enr_end-Enr_start) >= &enrdays.+&enrdaysaftind.;
		run;

	*END code for complex HOI; 

		data _TreatmentGroupForMatch (drop=adate enr_end enr_start EpisodeStartDt t4fupwashper washper);
		set _PtsMasterList;
		idt = _N_;
		age = floor((intck('month',Birth_Date,EpisodeEndDt) - (day(EpisodeEndDt) < day(Birth_Date))) / 12);
		run;

		%macro MatchPatientsForControl();

		    proc datasets library=work nolist nowarn;
		    delete _MatchingResults;        
		    quit;

		    * Get treatment minimum and maximum delivery age;
		    proc sql noprint;
		        select min(age)
		        into :MinAge	
		        from _TreatmentGroupForMatch;

		        select max(age)
		        into :MaxAge	
		        from _TreatmentGroupForMatch;
		    quit;

		    %put &MinAge. &MaxAge.;

		    * Create control datasets according to each age;
		    data 
		        %do i=&MinAge. %to &MaxAge.;
		            _control_&i.
		        %end;
		    ;
		    set dplocal._controlpopulation;
		        %do i=&MinAge. %to &MaxAge.;
		            if MinAge <= &i. <= MaxAge then output _control_&i.;
		        %end;
		    run;

		    * Loop on each age;
		    %do i=&MinAge. %to &MaxAge.;

		        *Match treatment with control;
		        proc sql noprint;
		            create table _MatchingResults_&i. as
		            select treat.IDT,
		                   contr.IDC,
		    	           ranuni(123457) as ToRandSelect
		            from _TreatmentGroupForMatch(keep=IndexDt IDT EpisodeEndDt Age where=(Age=&i.)) as treat,
		                 _control_&i. as contr
		            where treat.EpisodeEndDt+&enrdaysaftind. <= contr.Enr_End and contr.Enr_Start <= treat.EpisodeEndDt-&enrdays. and /*at least x ENRDAYS before deldate*/
						  contr.CStart <= treat.IndexDt <= contr.CEnd and											  /*start pregnancy during control period*/	
		                  floor((treat.EpisodeEndDt-contr.Birth_date)/365.25) = treat.Age							  /*Age at treated deldate = treated age*/
		            order IDT, ToRandSelect; 
		        quit;

		    	* Keep one control per treatment;
		        data _MatchingResults_&i.;
		        set _MatchingResults_&i.;
		        by IDT;
		        if first.IDT;
		        run;

		    	%IF %SYSFUNC(exist(_MatchingResults_&i.))=1 %THEN %DO;
		     	    proc datasets library=work nolist;
		            append base=_MatchingResults data=_MatchingResults_&i. FORCE;        
		            quit;
		    	%END;

		    %end;

		    proc datasets library=work nolist;
		    delete _matchingresults_: _control_:;
		    quit;

		%mend MatchPatientsForControl;

		*Do no run the matching macro if _TreatmentGroupForMatch is empty but create an empty table instead;
		%ISDATA(dataset=_TreatmentGroupForMatch);
		%IF %EVAL(&NOBS.>=1) %THEN %DO;
			%MatchPatientsForControl();
		%END;
		%ELSE %DO;	
			data _matchingresults;
			format idt idc torandselect best12.;
			call missing(idt, idc, torandselect);
			if _N_ = 0 then output;
			stop;
			run;
		%END;

		proc sql noprint;
		    Create Table _ControlGroup as
		    select contr.PatId,
		           contr.Birth_Date as Ctrl_birth_date,
				   treat.birth_date as treat_birth_date,
		           contr.Enr_Start,   
		           contr.Enr_End,    
				   contr.sex,
				   contr.race,
				   contr.hispanic,
				   contr.zip,
				   contr.zip_date,
		           treat.EpisodeEndDt,
		           treat.IndexDt,
		    	   treat.PregDurCode,
				   treat.ga_birth,
			  	   treat.Duration, 
		    	   treat.DelNum,		   
		    	   treat.AgeGroup,
				   treat.AgeGroupNum,
		      	   treat.Year,	    	
				   treat.month,
				   treat.indexlook,
				   treat.IndexDtLookEndDt,
				   treat.IndexDtLookStart,
		    	   treat.Has3Trim,
				   treat.ConcWashPer,
				   treat.PrePostInd,
				   (treat.EpisodeEndDt-contr.birth_date)/365.25 as Age
		    from dplocal._controlpopulation as contr,
		    	 _TreatmentGroupForMatch as treat,
		         _matchingresults as mres

		    where contr.idc = mres.idc and treat.idt = mres.idt;
		quit;

		*Process Zip Codes;
		%if "&geog." = "Y" %then %do;	
			proc sql noprint undo_policy=none;
				create table _controlgroup as
				select pts.*,
					   lkup.statecode,
					   lkup.hhs_region,
					   lkup.cb_region
				from _ControlGroup as pts
				left join infolder.&zipfile. as lkup
				on pts.zip = lkup.zip;
			quit;
		%end;

		data  _ControlGroup(drop=zip);
			format Enr_Start Enr_End DATE9. group $40.;
			length zip3 state hhs_reg cb_reg $7 zip_uncertain $1;
			set _ControlGroup;

			Group="&ITGROUP.";
			DelNum=_N_;	* To make sure patients used more than once in the matching process are considered distinct deliveries; 
			feventdt = .;

			%if "&geog." = "Y" %then %do;	
				if missing(zip) then do;
					zip3 = 'Missing';
					state = 'Missing';
					hhs_reg = 'Missing';
					cb_reg = 'Missing';
				end;
				else if missing(statecode) then do;
					zip3 = 'Invalid';
					state = 'Invalid';
					hhs_reg = 'Invalid';
					cb_reg = 'Invalid';
				end;
				else do;
					zip3 = substr(strip(zip),1,3);
					state = statecode;
					cb_reg = cb_region;
					hhs_reg = hhs_region;
					if statecode ne '' and missing(hhs_region) then do;
						hhs_reg = 'Other';
					end;
					if statecode ne '' and missing(cb_reg) then do;
						cb_reg = 'Other';
					end;
					end;
				if missing(zip_date) or EpisodeEndDt < zip_date then Zip_uncertain = 'Y';
				else zip_uncertain = 'N';

				drop statecode hhs_region cb_region;
			%end;
			%else %do;
				zip3 = '';
				state = '';
				cb_reg = '';
				hhs_reg = '';
				zip_uncertain = '';
			%end;
		run;

	%end;

	proc datasets nowarn noprint nolist lib=dplocal;
		delete control:;
	quit;

	%put NOTE: ******** END OF MACRO: ms_createnopreggroup v3.1 ********;

%mend ms_createnopreggroup;
