****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_create_censoring_table.sas  
*
* Created (mm/dd/yyyy): 10/13/2017
* Last modified: 12/28/2018
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: The macro computes aggregated reason for censoring
*
*  Program inputs:                                                                                   
*  	-
* 
*  Program outputs:                                                                                                                                       
*  	-
* 
*  PARAMETERS:        

*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  12/26/18		AP		Modifications for integration into QRP (removed DP stratified rows)
*
***************************************************************************************************;

%macro ms_t5_create_censoring_table();

	%put =====> MACRO CALLED: ms_t5_create_censoring_table v1.1;

	proc sql noprint;
	  select levelid1, levelid2 into: level1, :level2
	  from &tablefile.
	  where table = 'T15';
	quit;

	data agg_episdur_censor1;
		set agg_episdur_censor(keep=cens: episode: group runid level where=(level in("&level1", "&level2")));
			length percent_total percent_elig percent_dth percent_dpqry percent_episend percent_spec $10
                   censn_elig censn_dth censn_dpqry censn_episend censn_spec $15;

			if (level="&level1" | (level="&level2" & episodenum = 1));

			if episodenum = . then descr = 'Episodes';
			if episodenum  = 1 then descr = 'Patients';

			*combine qryend and dpend;
			if cens_qryend = 0 then cens_qryend = .;
			if cens_dpend = 0 then cens_dpend = .;
			cens_dpqry = coalesce(cens_qryend, cens_dpend);
			if cens_dpqry = . then cens_dpqry = 0;

			/* calculate the percents                     */
		    if episodes = 0 then do;
               percent_total ='-';
			   percent_elig  ='-';
			   percent_dth   ='-';
			   percent_dpqry ='-';
			   percent_episend='-';
			   percent_spec ='-';
			end;
		    else do;
               percent_total  = strip(put((episodes/episodes)*100,5.1));
			   percent_elig   = strip(put((cens_elig/episodes)*100,5.1));
			   percent_dth    = strip(put((cens_dth/episodes)*100,5.1));
			   percent_dpqry = strip(put((cens_dpqry/episodes)*100,5.1));
			   percent_episend = strip(put((cens_episend/episodes)*100,5.1));
			   percent_spec = strip(put((cens_spec/episodes)*100,5.1));
			end;

			count=strip(put(episodes, comma12.0));
		    			
			array tempn{*} censn_elig censn_dth censn_dpqry censn_episend censn_spec;
			array temp{*}  cens_elig cens_dth cens_dpqry cens_episend cens_spec;
			do i=1 to dim(temp);
			  tempn{i}=strip(put(temp{i}, comma12.0));
			end;
			
			*drop columns that are not relevant to the report;
			drop episodelength episodenum level cens_elig cens_dth cens_qryend cens_spec cens_dpend cens_episend i;
	run;

	/* Determine censoring criteria by group */

	/* taking care of censor_dth, censor_qryend and cencor_dpend columns */
	/* determine if columns can be dropped */
	%global displaydth displaydpqry displayspec;
	%let displaydth = N;
	%let displaydpqry = N;
	%let displayspec = N;

	%do grpcnt=1 %to &num_allgroups;
		
		%let censor_dth=;
		%let censor_qryend=;
		%let censor_dpend=;
		%let requester_defined_censor=;

		data _null_;
			set &groupfile.;
			if _n_ = &grpcnt. then do;
				call symputx("group", group);
		  	 	call symputx("runid", runid);
				call symputx("type5file", type5file);
		   		call symputx("requester_defined_censor", requester_defined_censor);
			end;
		run;

  		data _null_;
		   set infolder.&type5file.;
           if lowcase(group)="&group";	   
		   call symputx("censor_dth", lowcase(censor_dth));
		   call symputx("censor_qryend", lowcase(censor_qryend));
		   call symputx("censor_dpend", lowcase(censor_dpend));
		run;

		/*Any group is Y - set to Y*/
  		%if "&censor_dth" = "y" %then %do;
			%let displaydth = Y;
		%end;
		%if "&censor_qryend" = "y" | "&censor_dpend" = "y" %then %do; 
			%let displaydpqry = Y;
		%end;
		%if %length(&requester_defined_censor) > 0 | &requester_defined_censor ne  %str() %then %do; 
			%let displayspec = Y;
		%end;

		data _temp_censor&grpcnt.;
		   set agg_episdur_censor1;
		   if group ="&group"; 	
		   %if "&censor_dth" = "n" %then %do;
				censn_dth = '-';
				percent_dth =  '-';
		   %end;
		   %if "&censor_qryend" = "n" and "&censor_dpend" = "n" %then %do; 
				censn_dpqry = '-';
				percent_dpqry = '-';
			%end;
		    %if %length(&requester_defined_censor) = 0 | &requester_defined_censor = %str() %then %do; 
				censn_spec = '-';
				percent_spec = '-';
			%end;
		   ;
		run;
    %end;

	data agg_episdur_censor2;
	   set _temp_censor1-_temp_censor%cmpres(&num_allgroups.);
	run;

	/* add group order and label for reporting */
	proc sql noprint;
		create table censor_table as
		select x.*,
			y.exposure,
			y.order
		from agg_episdur_censor2 as x
		left join &groupfile. as y
		on x.group = y.group and x.runid = y.runid
		order by descr, order;
	quit;

	proc datasets noprint nowarn nolist lib=work;
		delete _temp_censor: agg_episdur_censor1 agg_episdur_censor2 header;
	quit;

	%put =====> END MACRO: ms_t5_create_censoring_table v1.1;

%mend ms_t5_create_censoring_table;
